#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the psort CLI tool."""

import argparse
import os
import unittest

from plaso.cli import psort_tool
from plaso.cli.helpers import interface as helpers_interface
from plaso.cli.helpers import manager as helpers_manager
from plaso.lib import errors
from plaso.output import interface as output_interface
from plaso.output import manager as output_manager

from tests import test_lib as shared_test_lib
from tests.cli import test_lib


class TestInputReader(object):
  """Test input reader."""

  def __init__(self):
    """Initialize the reader."""
    super(TestInputReader, self).__init__()
    self.read_called = False

  def Read(self):
    """Mock a read operation by user."""
    self.read_called = True
    return u'foobar'


class TestOutputModuleArgumentHelper(helpers_interface.ArgumentsHelper):
  """Test argument helper for the test output module."""

  NAME = u'test_missing'

  @classmethod
  def AddArguments(cls, argument_group):
    """Mock the add argument section."""
    pass

  # pylint: disable=arguments-differ
  @classmethod
  def ParseOptions(cls, options, output_module):
    """Provide a test parse options section."""
    if not isinstance(output_module, TestOutputModuleMissingParameters):
      raise errors.BadConfigObject((
          u'Output module is not an instance of '
          u'TestOutputModuleMissingParameters'))

    missing = getattr(options, u'missing', None)
    if missing:
      output_module.SetMissingValue(u'missing', missing)

    parameters = getattr(options, u'parameters', None)
    if parameters:
      output_module.SetMissingValue(u'parameters', parameters)


class TestOutputModuleMissingParameters(output_interface.LinearOutputModule):
  """Test output module that is missing some parameters."""

  NAME = u'test_missing'

  _HEADER = (
      u'date,time,timezone,MACB,source,sourcetype,type,user,host,'
      u'short,desc,version,filename,inode,notes,format,extra\n')

  # For test purpose assign these as class attributes.
  missing = None
  parameters = None

  def GetMissingArguments(self):
    """Return a list of missing parameters."""
    missing_parameters = []
    if self.missing is None:
      missing_parameters.append(u'missing')

    if self.parameters is None:
      missing_parameters.append(u'parameters')

    return missing_parameters

  @classmethod
  def SetMissingValue(cls, attribute, value):
    """Set missing value."""
    setattr(cls, attribute, value)

  def WriteEventBody(self, event):
    """Writes the body of an event object to the output.

    Args:
      event (EventObject): event.
    """
    message, _ = self._output_mediator.GetFormattedMessages(event)
    source_short, source_long = self._output_mediator.GetFormattedSources(event)
    self._WriteLine(u'{0:s}/{1:s} {2:s}\n'.format(
        source_short, source_long, message))

  def WriteHeader(self):
    """Writes the header to the output."""
    self._WriteLine(self._HEADER)


class PsortToolTest(test_lib.CLIToolTestCase):
  """Tests for the psort tool."""

  _EXPECTED_PROCESSING_OPTIONS = u'\n'.join([
      (u'usage: psort_test.py [--disable_zeromq] '
       u'[--temporary_directory DIRECTORY]'),
      u'                     [--worker-memory-limit SIZE]',
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      u'  --disable_zeromq, --disable-zeromq',
      (u'                        Disable queueing using ZeroMQ. A '
       u'Multiprocessing queue'),
      u'                        will be used instead.',
      u'  --temporary_directory DIRECTORY, --temporary-directory DIRECTORY',
      (u'                        Path to the directory that should be used to '
       u'store'),
      u'                        temporary files created during processing.',
      u'  --worker-memory-limit SIZE, --worker_memory_limit SIZE',
      (u'                        Maximum amount of memory a worker process is '
       u'allowed'),
      u'                        to consume. [defaults to 2 GiB]',
      u''])

  # TODO: add test for _CreateOutputModule.
  # TODO: add test for _FormatStatusTableRow.
  # TODO: add test for _GetAnalysisPlugins.
  # TODO: add test for _ParseAnalysisPluginOptions.
  # TODO: add test for _ParseProcessingOptions.
  # TODO: add test for _ParseInformationalOptions.
  # TODO: add test for _PrintStatusHeader.
  # TODO: add test for _PrintStatusUpdate.
  # TODO: add test for _PrintStatusUpdateStream.

  def testAddProcessingOptions(self):
    """Tests the AddProcessingOptions function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'psort_test.py',
        description=u'Test argument parser.', add_help=False,
        formatter_class=test_lib.SortedArgumentsHelpFormatter)

    test_tool = psort_tool.PsortTool()
    test_tool.AddProcessingOptions(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_PROCESSING_OPTIONS)

  def testListLanguageIdentifiers(self):
    """Tests the ListLanguageIdentifiers function."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psort_tool.PsortTool(output_writer=output_writer)

    test_tool.ListLanguageIdentifiers()

    output = output_writer.ReadOutput()

    number_of_tables = 0
    lines = []
    for line in output.split(b'\n'):
      line = line.strip()
      lines.append(line)

      if line.startswith(b'*****') and line.endswith(b'*****'):
        number_of_tables += 1

    self.assertIn(u'Language identifiers', lines[1])

    lines = frozenset(lines)

    self.assertEqual(number_of_tables, 1)

    expected_line = b'en : English'
    self.assertIn(expected_line, lines)

  def testParseArguments(self):
    """Tests the ParseArguments function."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psort_tool.PsortTool(output_writer=output_writer)

    result = test_tool.ParseArguments()
    self.assertFalse(result)

    # TODO: check output.
    # TODO: improve test coverage.

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psort_tool.PsortTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.output_format = u'null'
    options.storage_file = self._GetTestFilePath([u'psort_test.json.plaso'])

    test_tool.ParseOptions(options)

    options = test_lib.TestOptions()

    with self.assertRaises(errors.BadConfigOption):
      test_tool.ParseOptions(options)

    options = test_lib.TestOptions()
    options.storage_file = self._GetTestFilePath([u'psort_test.json.plaso'])

    with self.assertRaises(errors.BadConfigOption):
      test_tool.ParseOptions(options)

    # TODO: improve test coverage.

  def testProcessStorageWithMissingParameters(self):
    """Tests the ProcessStorage function with parameters missing."""
    input_reader = TestInputReader()
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psort_tool.PsortTool(
        input_reader=input_reader, output_writer=output_writer)

    options = test_lib.TestOptions()
    options.storage_file = self._GetTestFilePath([u'psort_test.json.plaso'])
    options.output_format = u'test_missing'

    output_manager.OutputManager.RegisterOutput(
        TestOutputModuleMissingParameters)
    helpers_manager.ArgumentHelperManager.RegisterHelper(
        TestOutputModuleArgumentHelper)

    lines = []
    with shared_test_lib.TempDirectory() as temp_directory:
      temp_file_name = os.path.join(temp_directory, u'output.txt')
      options.write = temp_file_name

      test_tool.ParseOptions(options)
      test_tool.ProcessStorage()

      with open(temp_file_name, 'rb') as file_object:
        for line in file_object.readlines():
          lines.append(line.strip())

    self.assertTrue(input_reader.read_called)
    self.assertEqual(TestOutputModuleMissingParameters.missing, u'foobar')
    self.assertEqual(TestOutputModuleMissingParameters.parameters, u'foobar')

    expected_line = (
        u'FILE/OS Metadata Modification Time OS:/tmp/test/test_data/syslog '
        u'Type: file')
    self.assertIn(expected_line, lines)

    output_manager.OutputManager.DeregisterOutput(
        TestOutputModuleMissingParameters)
    helpers_manager.ArgumentHelperManager.DeregisterHelper(
        TestOutputModuleArgumentHelper)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Basic Security Module Parser."""

import binascii
import logging
import os
import socket

import construct

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import errors
from plaso.lib import timelib
from plaso.parsers import interface
from plaso.parsers import manager
from plaso.unix import bsmtoken

import pytz  # pylint: disable=wrong-import-order


__author__ = 'Joaquin Moreno Garijo (Joaquin.MorenoGarijo.2013@live.rhul.ac.uk)'


# Note that we're using Array and a helper function here instead of
# PascalString because the latter seems to break pickling on Windows.

def _BSMTokenGetLength(context):
  """Construct context parser helper function to replace lambda."""
  return context.length


# Note that we're using RepeatUntil and a helper function here instead of
# CString because the latter seems to break pickling on Windows.

def _BSMTokenIsEndOfString(value, unused_context):
  """Construct context parser helper function to replace lambda."""
  return value == b'\x00'


# Note that we're using Switch and a helper function here instead of
# IfThenElse because the latter seems to break pickling on Windows.

def _BSMTokenGetNetType(context):
  """Construct context parser helper function to replace lambda."""
  return context.net_type


def _BSMTokenGetSocketDomain(context):
  """Construct context parser helper function to replace lambda."""
  return context.socket_domain


class BSMEventData(events.EventData):
  """Generic BSM event data.

  Attributes:
    event_type (str): text with ID that represents the type of the event.
    extra_tokens (dict): event extra tokens.
    record_length (int): record length in bytes (trailer number).
  """

  DATA_TYPE = u'bsm:event'

  def __init__(self):
    """Initializes event data."""
    super(BSMEventData, self).__init__(data_type=self.DATA_TYPE)
    self.event_type = None
    self.extra_tokens = None
    self.record_length = None


class MacBSMEventData(events.EventData):
  """Mac OS X BSM event data.

  Attributes:
    event_type (str): text with ID that represents the type of the event.
    extra_tokens (dict): event extra tokens.
    record_length (int): record length in bytes (trailer number).
    return_value (str): processed return value and exit status.
  """

  DATA_TYPE = u'mac:bsm:event'

  def __init__(self):
    """Initializes event data."""
    super(MacBSMEventData, self).__init__(data_type=self.DATA_TYPE)
    self.event_type = None
    self.extra_tokens = None
    self.record_length = None
    self.return_value = None


class BSMParser(interface.FileObjectParser):
  """Parser for BSM files."""

  NAME = u'bsm_log'
  DESCRIPTION = u'Parser for BSM log files.'

  # BSM supported version (0x0b = 11).
  AUDIT_HEADER_VERSION = 11

  # Magic Trail Header.
  BSM_TOKEN_TRAILER_MAGIC = b'b105'

  # IP Version constants.
  AU_IPv4 = 4
  AU_IPv6 = 16

  IPV4_STRUCT = construct.UBInt32(u'ipv4')

  IPV6_STRUCT = construct.Struct(
      u'ipv6',
      construct.UBInt64(u'high'),
      construct.UBInt64(u'low'))

  # Tested structures.
  # INFO: I have ommited the ID in the structures declaration.
  #       I used the BSM_TYPE first to read the ID, and then, the structure.
  # Tokens always start with an ID value that identifies their token
  # type and subsequent structure.
  _BSM_TOKEN = construct.UBInt8(u'token_id')

  # Data type structures.
  BSM_TOKEN_DATA_CHAR = construct.String(u'value', 1)
  BSM_TOKEN_DATA_SHORT = construct.UBInt16(u'value')
  BSM_TOKEN_DATA_INTEGER = construct.UBInt32(u'value')

  # Common structure used by other structures.
  # audit_uid: integer, uid that generates the entry.
  # effective_uid: integer, the permission user used.
  # effective_gid: integer, the permission group used.
  # real_uid: integer, user id of the user that execute the process.
  # real_gid: integer, group id of the group that execute the process.
  # pid: integer, identification number of the process.
  # session_id: unknown, need research.
  BSM_TOKEN_SUBJECT_SHORT = construct.Struct(
      u'subject_data',
      construct.UBInt32(u'audit_uid'),
      construct.UBInt32(u'effective_uid'),
      construct.UBInt32(u'effective_gid'),
      construct.UBInt32(u'real_uid'),
      construct.UBInt32(u'real_gid'),
      construct.UBInt32(u'pid'),
      construct.UBInt32(u'session_id'))

  # Common structure used by other structures.
  # Identify the kind of inet (IPv4 or IPv6)
  # TODO: instead of 16, AU_IPv6 must be used.
  BSM_IP_TYPE_SHORT = construct.Struct(
      u'bsm_ip_type_short',
      construct.UBInt32(u'net_type'),
      construct.Switch(
          u'ip_addr',
          _BSMTokenGetNetType,
          {16: IPV6_STRUCT},
          default=IPV4_STRUCT))

  # Initial fields structure used by header structures.
  # length: integer, the length of the entry, equal to trailer (doc: length).
  # version: integer, version of BSM (AUDIT_HEADER_VERSION).
  # event_type: integer, the type of event (/etc/security/audit_event).
  # modifier: integer, unknown, need research (It is always 0).
  BSM_HEADER = construct.Struct(
      u'bsm_header',
      construct.UBInt32(u'length'),
      construct.UBInt8(u'version'),
      construct.UBInt16(u'event_type'),
      construct.UBInt16(u'modifier'))

  # First token of one entry.
  # timestamp: unsigned integer, number of seconds since
  #            January 1, 1970 00:00:00 UTC.
  # microseconds: unsigned integer, number of micro seconds.
  BSM_HEADER32 = construct.Struct(
      u'bsm_header32',
      BSM_HEADER,
      construct.UBInt32(u'timestamp'),
      construct.UBInt32(u'microseconds'))

  BSM_HEADER64 = construct.Struct(
      u'bsm_header64',
      BSM_HEADER,
      construct.UBInt64(u'timestamp'),
      construct.UBInt64(u'microseconds'))

  BSM_HEADER32_EX = construct.Struct(
      u'bsm_header32_ex',
      BSM_HEADER,
      BSM_IP_TYPE_SHORT,
      construct.UBInt32(u'timestamp'),
      construct.UBInt32(u'microseconds'))

  # Token TEXT, provides extra information.
  BSM_TOKEN_TEXT = construct.Struct(
      u'bsm_token_text',
      construct.UBInt16(u'length'),
      construct.Array(_BSMTokenGetLength, construct.UBInt8(u'text')))

  # Path of the executable.
  BSM_TOKEN_PATH = BSM_TOKEN_TEXT

  # Identified the end of the record (follow by TRAILER).
  # status: integer that identifies the status of the exit (BSM_ERRORS).
  # return: returned value from the operation.
  BSM_TOKEN_RETURN32 = construct.Struct(
      u'bsm_token_return32',
      construct.UBInt8(u'status'),
      construct.UBInt32(u'return_value'))

  BSM_TOKEN_RETURN64 = construct.Struct(
      u'bsm_token_return64',
      construct.UBInt8(u'status'),
      construct.UBInt64(u'return_value'))

  # Identified the number of bytes that was written.
  # magic: 2 bytes that identifies the TRAILER (BSM_TOKEN_TRAILER_MAGIC).
  # length: integer that has the number of bytes from the entry size.
  BSM_TOKEN_TRAILER = construct.Struct(
      u'bsm_token_trailer',
      construct.UBInt16(u'magic'),
      construct.UBInt32(u'record_length'))

  # A 32-bits argument.
  # num_arg: the number of the argument.
  # name_arg: the argument's name.
  # text: the string value of the argument.
  BSM_TOKEN_ARGUMENT32 = construct.Struct(
      u'bsm_token_argument32',
      construct.UBInt8(u'num_arg'),
      construct.UBInt32(u'name_arg'),
      construct.UBInt16(u'length'),
      construct.Array(_BSMTokenGetLength, construct.UBInt8(u'text')))

  # A 64-bits argument.
  # num_arg: integer, the number of the argument.
  # name_arg: text, the argument's name.
  # text: the string value of the argument.
  BSM_TOKEN_ARGUMENT64 = construct.Struct(
      u'bsm_token_argument64',
      construct.UBInt8(u'num_arg'),
      construct.UBInt64(u'name_arg'),
      construct.UBInt16(u'length'),
      construct.Array(_BSMTokenGetLength, construct.UBInt8(u'text')))

  # Identify an user.
  # terminal_id: unknown, research needed.
  # terminal_addr: unknown, research needed.
  BSM_TOKEN_SUBJECT32 = construct.Struct(
      u'bsm_token_subject32',
      BSM_TOKEN_SUBJECT_SHORT,
      construct.UBInt32(u'terminal_port'),
      IPV4_STRUCT)

  # Identify an user using a extended Token.
  # terminal_port: unknown, need research.
  # net_type: unknown, need research.
  BSM_TOKEN_SUBJECT32_EX = construct.Struct(
      u'bsm_token_subject32_ex',
      BSM_TOKEN_SUBJECT_SHORT,
      construct.UBInt32(u'terminal_port'),
      BSM_IP_TYPE_SHORT)

  # au_to_opaque // AUT_OPAQUE
  BSM_TOKEN_OPAQUE = BSM_TOKEN_TEXT

  # au_to_seq // AUT_SEQ
  BSM_TOKEN_SEQUENCE = BSM_TOKEN_DATA_INTEGER

  # Program execution with options.
  # For each argument we are going to have a string+ "\x00".
  # Example: [00 00 00 02][41 42 43 00 42 42 00]
  #          2 Arguments, Arg1: [414243] Arg2: [4242].
  BSM_TOKEN_EXEC_ARGUMENTS = construct.UBInt32(u'number_arguments')

  BSM_TOKEN_EXEC_ARGUMENT = construct.Struct(
      u'bsm_token_exec_argument',
      construct.RepeatUntil(
          _BSMTokenIsEndOfString, construct.StaticField("text", 1)))

  # au_to_in_addr // AUT_IN_ADDR:
  BSM_TOKEN_ADDR = IPV4_STRUCT

  # au_to_in_addr_ext // AUT_IN_ADDR_EX:
  BSM_TOKEN_ADDR_EXT = construct.Struct(
      u'bsm_token_addr_ext',
      construct.UBInt32(u'net_type'),
      IPV6_STRUCT)

  # au_to_ip // AUT_IP:
  # TODO: parse this header in the correct way.
  BSM_TOKEN_IP = construct.String(u'binary_ipv4_add', 20)

  # au_to_ipc // AUT_IPC:
  BSM_TOKEN_IPC = construct.Struct(
      u'bsm_token_ipc',
      construct.UBInt8(u'object_type'),
      construct.UBInt32(u'object_id'))

  # au_to_ipc_perm // au_to_ipc_perm
  BSM_TOKEN_IPC_PERM = construct.Struct(
      u'bsm_token_ipc_perm',
      construct.UBInt32(u'user_id'),
      construct.UBInt32(u'group_id'),
      construct.UBInt32(u'creator_user_id'),
      construct.UBInt32(u'creator_group_id'),
      construct.UBInt32(u'access_mode'),
      construct.UBInt32(u'slot_seq'),
      construct.UBInt32(u'key'))

  # au_to_iport // AUT_IPORT:
  BSM_TOKEN_PORT = construct.UBInt16(u'port_number')

  # au_to_file // AUT_OTHER_FILE32:
  BSM_TOKEN_FILE = construct.Struct(
      u'bsm_token_file',
      construct.UBInt32(u'timestamp'),
      construct.UBInt32(u'microseconds'),
      construct.UBInt16(u'length'),
      construct.Array(_BSMTokenGetLength, construct.UBInt8(u'text')))

  # au_to_subject64 // AUT_SUBJECT64:
  BSM_TOKEN_SUBJECT64 = construct.Struct(
      u'bsm_token_subject64',
      BSM_TOKEN_SUBJECT_SHORT,
      construct.UBInt64(u'terminal_port'),
      IPV4_STRUCT)

  # au_to_subject64_ex // AU_IPv4:
  BSM_TOKEN_SUBJECT64_EX = construct.Struct(
      u'bsm_token_subject64_ex',
      BSM_TOKEN_SUBJECT_SHORT,
      construct.UBInt32(u'terminal_port'),
      construct.UBInt32(u'terminal_type'),
      BSM_IP_TYPE_SHORT)

  # au_to_process32 // AUT_PROCESS32:
  BSM_TOKEN_PROCESS32 = construct.Struct(
      u'bsm_token_process32',
      BSM_TOKEN_SUBJECT_SHORT,
      construct.UBInt32(u'terminal_port'),
      IPV4_STRUCT)

  # au_to_process64 // AUT_PROCESS32:
  BSM_TOKEN_PROCESS64 = construct.Struct(
      u'bsm_token_process64',
      BSM_TOKEN_SUBJECT_SHORT,
      construct.UBInt64(u'terminal_port'),
      IPV4_STRUCT)

  # au_to_process32_ex // AUT_PROCESS32_EX:
  BSM_TOKEN_PROCESS32_EX = construct.Struct(
      u'bsm_token_process32_ex',
      BSM_TOKEN_SUBJECT_SHORT,
      construct.UBInt32(u'terminal_port'),
      BSM_IP_TYPE_SHORT)

  # au_to_process64_ex // AUT_PROCESS64_EX:
  BSM_TOKEN_PROCESS64_EX = construct.Struct(
      u'bsm_token_process64_ex',
      BSM_TOKEN_SUBJECT_SHORT,
      construct.UBInt64(u'terminal_port'),
      BSM_IP_TYPE_SHORT)

  # au_to_sock_inet32 // AUT_SOCKINET32:
  BSM_TOKEN_AUT_SOCKINET32 = construct.Struct(
      u'bsm_token_aut_sockinet32',
      construct.UBInt16(u'net_type'),
      construct.UBInt16(u'port_number'),
      IPV4_STRUCT)

  # Info: checked against the source code of XNU, but not against
  #       real BSM file.
  BSM_TOKEN_AUT_SOCKINET128 = construct.Struct(
      u'bsm_token_aut_sockinet128',
      construct.UBInt16(u'net_type'),
      construct.UBInt16(u'port_number'),
      IPV6_STRUCT)

  INET6_ADDR_TYPE = construct.Struct(
      u'addr_type',
      construct.UBInt16(u'ip_type'),
      construct.UBInt16(u'source_port'),
      construct.UBInt64(u'saddr_high'),
      construct.UBInt64(u'saddr_low'),
      construct.UBInt16(u'destination_port'),
      construct.UBInt64(u'daddr_high'),
      construct.UBInt64(u'daddr_low'))

  INET4_ADDR_TYPE = construct.Struct(
      u'addr_type',
      construct.UBInt16(u'ip_type'),
      construct.UBInt16(u'source_port'),
      construct.UBInt32(u'source_address'),
      construct.UBInt16(u'destination_port'),
      construct.UBInt32(u'destination_address'))

  # au_to_socket_ex // AUT_SOCKET_EX
  # TODO: Change the 26 for unixbsm.BSM_PROTOCOLS.INET6.
  BSM_TOKEN_AUT_SOCKINET32_EX = construct.Struct(
      u'bsm_token_aut_sockinet32_ex',
      construct.UBInt16(u'socket_domain'),
      construct.UBInt16(u'socket_type'),
      construct.Switch(
          u'structure_addr_port',
          _BSMTokenGetSocketDomain,
          {26: INET6_ADDR_TYPE},
          default=INET4_ADDR_TYPE))

  # au_to_sock_unix // AUT_SOCKUNIX
  BSM_TOKEN_SOCKET_UNIX = construct.Struct(
      u'bsm_token_au_to_sock_unix',
      construct.UBInt16(u'family'),
      construct.RepeatUntil(
          _BSMTokenIsEndOfString,
          construct.StaticField("path", 1)))

  # au_to_data // au_to_data
  # how to print: bsmtoken.BSM_TOKEN_DATA_PRINT.
  # type: bsmtoken.BSM_TOKEN_DATA_TYPE.
  # unit_count: number of type values.
  # BSM_TOKEN_DATA has a end field = type * unit_count
  BSM_TOKEN_DATA = construct.Struct(
      u'bsm_token_data',
      construct.UBInt8(u'how_to_print'),
      construct.UBInt8(u'data_type'),
      construct.UBInt8(u'unit_count'))

  # au_to_attr32 // AUT_ATTR32
  BSM_TOKEN_ATTR32 = construct.Struct(
      u'bsm_token_attr32',
      construct.UBInt32(u'file_mode'),
      construct.UBInt32(u'uid'),
      construct.UBInt32(u'gid'),
      construct.UBInt32(u'file_system_id'),
      construct.UBInt64(u'file_system_node_id'),
      construct.UBInt32(u'device'))

  # au_to_attr64 // AUT_ATTR64
  BSM_TOKEN_ATTR64 = construct.Struct(
      u'bsm_token_attr64',
      construct.UBInt32(u'file_mode'),
      construct.UBInt32(u'uid'),
      construct.UBInt32(u'gid'),
      construct.UBInt32(u'file_system_id'),
      construct.UBInt64(u'file_system_node_id'),
      construct.UBInt64(u'device'))

  # au_to_exit // AUT_EXIT
  BSM_TOKEN_EXIT = construct.Struct(
      u'bsm_token_exit',
      construct.UBInt32(u'status'),
      construct.UBInt32(u'return_value'))

  # au_to_newgroups // AUT_NEWGROUPS
  # INFO: we must read BSM_TOKEN_DATA_INTEGER for each group.
  BSM_TOKEN_GROUPS = construct.UBInt16(u'group_number')

  # au_to_exec_env == au_to_exec_args
  BSM_TOKEN_EXEC_ENV = BSM_TOKEN_EXEC_ARGUMENTS

  # au_to_zonename //AUT_ZONENAME
  BSM_TOKEN_ZONENAME = BSM_TOKEN_TEXT

  # Token ID.
  # List of valid Token_ID.
  # Token_ID -> (NAME_STRUCTURE, STRUCTURE)
  # Only the checked structures are been added to the valid structures lists.
  _BSM_TOKEN_TYPES = {
      17: (u'BSM_TOKEN_FILE', BSM_TOKEN_FILE),
      19: (u'BSM_TOKEN_TRAILER', BSM_TOKEN_TRAILER),
      20: (u'BSM_HEADER32', BSM_HEADER32),
      21: (u'BSM_HEADER64', BSM_HEADER64),
      33: (u'BSM_TOKEN_DATA', BSM_TOKEN_DATA),
      34: (u'BSM_TOKEN_IPC', BSM_TOKEN_IPC),
      35: (u'BSM_TOKEN_PATH', BSM_TOKEN_PATH),
      36: (u'BSM_TOKEN_SUBJECT32', BSM_TOKEN_SUBJECT32),
      38: (u'BSM_TOKEN_PROCESS32', BSM_TOKEN_PROCESS32),
      39: (u'BSM_TOKEN_RETURN32', BSM_TOKEN_RETURN32),
      40: (u'BSM_TOKEN_TEXT', BSM_TOKEN_TEXT),
      41: (u'BSM_TOKEN_OPAQUE', BSM_TOKEN_OPAQUE),
      42: (u'BSM_TOKEN_ADDR', BSM_TOKEN_ADDR),
      43: (u'BSM_TOKEN_IP', BSM_TOKEN_IP),
      44: (u'BSM_TOKEN_PORT', BSM_TOKEN_PORT),
      45: (u'BSM_TOKEN_ARGUMENT32', BSM_TOKEN_ARGUMENT32),
      47: (u'BSM_TOKEN_SEQUENCE', BSM_TOKEN_SEQUENCE),
      96: (u'BSM_TOKEN_ZONENAME', BSM_TOKEN_ZONENAME),
      113: (u'BSM_TOKEN_ARGUMENT64', BSM_TOKEN_ARGUMENT64),
      114: (u'BSM_TOKEN_RETURN64', BSM_TOKEN_RETURN64),
      116: (u'BSM_HEADER32_EX', BSM_HEADER32_EX),
      119: (u'BSM_TOKEN_PROCESS64', BSM_TOKEN_PROCESS64),
      122: (u'BSM_TOKEN_SUBJECT32_EX', BSM_TOKEN_SUBJECT32_EX),
      127: (u'BSM_TOKEN_AUT_SOCKINET32_EX', BSM_TOKEN_AUT_SOCKINET32_EX),
      128: (u'BSM_TOKEN_AUT_SOCKINET32', BSM_TOKEN_AUT_SOCKINET32)}

  # Untested structures.
  # When not tested structure is found, we try to parse using also
  # these structures.
  BSM_TYPE_LIST_NOT_TESTED = {
      49: (u'BSM_TOKEN_ATTR', BSM_TOKEN_ATTR32),
      50: (u'BSM_TOKEN_IPC_PERM', BSM_TOKEN_IPC_PERM),
      52: (u'BSM_TOKEN_GROUPS', BSM_TOKEN_GROUPS),
      59: (u'BSM_TOKEN_GROUPS', BSM_TOKEN_GROUPS),
      60: (u'BSM_TOKEN_EXEC_ARGUMENTS', BSM_TOKEN_EXEC_ARGUMENTS),
      61: (u'BSM_TOKEN_EXEC_ENV', BSM_TOKEN_EXEC_ENV),
      62: (u'BSM_TOKEN_ATTR32', BSM_TOKEN_ATTR32),
      82: (u'BSM_TOKEN_EXIT', BSM_TOKEN_EXIT),
      115: (u'BSM_TOKEN_ATTR64', BSM_TOKEN_ATTR64),
      117: (u'BSM_TOKEN_SUBJECT64', BSM_TOKEN_SUBJECT64),
      123: (u'BSM_TOKEN_PROCESS32_EX', BSM_TOKEN_PROCESS32_EX),
      124: (u'BSM_TOKEN_PROCESS64_EX', BSM_TOKEN_PROCESS64_EX),
      125: (u'BSM_TOKEN_SUBJECT64_EX', BSM_TOKEN_SUBJECT64_EX),
      126: (u'BSM_TOKEN_ADDR_EXT', BSM_TOKEN_ADDR_EXT),
      129: (u'BSM_TOKEN_AUT_SOCKINET128', BSM_TOKEN_AUT_SOCKINET128),
      130: (u'BSM_TOKEN_SOCKET_UNIX', BSM_TOKEN_SOCKET_UNIX)}

  MESSAGE_CAN_NOT_SAVE = (
      u'Plaso: some tokens from this entry can not be saved. Entry at 0x{0:X} '
      u'with unknown token id "0x{1:X}".')

  # BSM token types:
  # https://github.com/openbsm/openbsm/blob/master/sys/bsm/audit_record.h
  _BSM_TOKEN_TYPE_ARGUMENT32 = 45
  _BSM_TOKEN_TYPE_ARGUMENT64 = 113
  _BSM_TOKEN_TYPE_ATTR = 49
  _BSM_TOKEN_TYPE_ATTR32 = 62
  _BSM_TOKEN_TYPE_ATTR64 = 115
  _BSM_TOKEN_TYPE_EXEC_ARGUMENTS = 60
  _BSM_TOKEN_TYPE_EXEC_ENV = 61
  _BSM_TOKEN_TYPE_EXIT = 82
  _BSM_TOKEN_TYPE_HEADER32 = 20
  _BSM_TOKEN_TYPE_HEADER32_EX = 116
  _BSM_TOKEN_TYPE_HEADER64 = 21
  _BSM_TOKEN_TYPE_PATH = 35
  _BSM_TOKEN_TYPE_PROCESS32 = 38
  _BSM_TOKEN_TYPE_PROCESS32_EX = 123
  _BSM_TOKEN_TYPE_PROCESS64 = 119
  _BSM_TOKEN_TYPE_PROCESS64_EX = 124
  _BSM_TOKEN_TYPE_RETURN32 = 39
  _BSM_TOKEN_TYPE_RETURN64 = 114
  _BSM_TOKEN_TYPE_SUBJECT32 = 36
  _BSM_TOKEN_TYPE_SUBJECT32_EX = 122
  _BSM_TOKEN_TYPE_SUBJECT64 = 117
  _BSM_TOKEN_TYPE_SUBJECT64_EX = 125
  _BSM_TOKEN_TYPE_TEXT = 40
  _BSM_TOKEN_TYPE_ZONENAME = 96

  _BSM_ARGUMENT_TOKEN_TYPES = (
      _BSM_TOKEN_TYPE_ARGUMENT32,
      _BSM_TOKEN_TYPE_ARGUMENT64)

  _BSM_ATTR_TOKEN_TYPES = (
      _BSM_TOKEN_TYPE_ATTR,
      _BSM_TOKEN_TYPE_ATTR32,
      _BSM_TOKEN_TYPE_ATTR64)

  _BSM_EXEV_TOKEN_TYPES = (
      _BSM_TOKEN_TYPE_EXEC_ARGUMENTS,
      _BSM_TOKEN_TYPE_EXEC_ENV)

  _BSM_HEADER_TOKEN_TYPES = (
      _BSM_TOKEN_TYPE_HEADER32,
      _BSM_TOKEN_TYPE_HEADER32_EX,
      _BSM_TOKEN_TYPE_HEADER64)

  _BSM_PROCESS_TOKEN_TYPES = (
      _BSM_TOKEN_TYPE_PROCESS32,
      _BSM_TOKEN_TYPE_PROCESS64)

  _BSM_PROCESS_EX_TOKEN_TYPES = (
      _BSM_TOKEN_TYPE_PROCESS32_EX,
      _BSM_TOKEN_TYPE_PROCESS64_EX)

  _BSM_RETURN_TOKEN_TYPES = (
      _BSM_TOKEN_TYPE_EXIT,
      _BSM_TOKEN_TYPE_RETURN32,
      _BSM_TOKEN_TYPE_RETURN64)

  _BSM_SUBJECT_TOKEN_TYPES = (
      _BSM_TOKEN_TYPE_SUBJECT32,
      _BSM_TOKEN_TYPE_SUBJECT64)

  _BSM_SUBJECT_EX_TOKEN_TYPES = (
      _BSM_TOKEN_TYPE_SUBJECT32_EX,
      _BSM_TOKEN_TYPE_SUBJECT64_EX)

  _BSM_UTF8_BYTE_ARRAY_TOKEN_TYPES = (
      _BSM_TOKEN_TYPE_PATH,
      _BSM_TOKEN_TYPE_TEXT,
      _BSM_TOKEN_TYPE_ZONENAME)

  def __init__(self):
    """Initializes a parser object."""
    super(BSMParser, self).__init__()
    # Create the dictionary with all token IDs: tested and untested.
    self._bsm_type_list_all = self._BSM_TOKEN_TYPES.copy()
    self._bsm_type_list_all.update(self.BSM_TYPE_LIST_NOT_TESTED)

  def _CopyByteArrayToBase16String(self, byte_array):
    """Copies a byte array into a base-16 encoded Unicode string.

    Args:
      byte_array (bytes): A byte array.

    Returns:
      str: a base-16 encoded Unicode string.
    """
    return u''.join([u'{0:02x}'.format(byte) for byte in byte_array])

  def _CopyUtf8ByteArrayToString(self, byte_array):
    """Copies a UTF-8 encoded byte array into a Unicode string.

    Args:
      byte_array (bytes): A byte array containing an UTF-8 encoded string.

    Returns:
      str: A Unicode string.
    """
    byte_stream = b''.join(map(chr, byte_array))

    try:
      string = byte_stream.decode(u'utf-8')
    except UnicodeDecodeError:
      logging.warning(u'Unable to decode UTF-8 formatted byte array.')
      string = byte_stream.decode(u'utf-8', errors=u'ignore')

    string, _, _ = string.partition(b'\x00')
    return string

  def _IPv4Format(self, address):
    """Formats an IPv4 address as a human readable string.

    Args:
      address (int): IPv4 address.

    Returns:
      str: human readable string of IPv4 address in 4 octet representation:
          "1.2.3.4".
    """
    ipv4_string = self.IPV4_STRUCT.build(address)
    return socket.inet_ntoa(ipv4_string)

  def _IPv6Format(self, high, low):
    """Formats an IPv6 address as a human readable string.

    Args:
      high (int): upper 64-bit part of the IPv6 address.
      low (int): lower 64-bit part of the IPv6 address.

    Returns:
      str: human readable string of IPv6 address.
    """
    ipv6_string = self.IPV6_STRUCT.build(
        construct.Container(high=high, low=low))
    # socket.inet_ntop not supported in Windows.
    if hasattr(socket, u'inet_ntop'):
      return socket.inet_ntop(socket.AF_INET6, ipv6_string)

    # TODO: this approach returns double "::", illegal IPv6 addr.
    str_address = binascii.hexlify(ipv6_string)
    address = []
    blank = False
    for pos in range(0, len(str_address), 4):
      if str_address[pos:pos + 4] == u'0000':
        if not blank:
          address.append(u'')
          blank = True
      else:
        blank = False
        address.append(str_address[pos:pos + 4].lstrip(u'0'))
    return u':'.join(address)

  def _ParseBSMEvent(self, parser_mediator, file_object):
    """Parses a BSM entry (BSMEvent) from the file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Returns:
      bool: True if the BSM entry was parsed.
    """
    record_start_offset = file_object.tell()

    try:
      token_type = self._BSM_TOKEN.parse_stream(file_object)
    except (IOError, construct.FieldError) as exception:
      parser_mediator.ProduceExtractionError((
          u'unable to parse BSM token type at offset: 0x{0:08x} with error: '
          u'{1:s}.').format(record_start_offset, exception))
      return False

    if token_type not in self._BSM_HEADER_TOKEN_TYPES:
      parser_mediator.ProduceExtractionError(
          u'unsupported token type: {0:d} at offset: 0x{1:08x}.'.format(
              token_type, record_start_offset))
      # TODO: if it is a Mac OS X, search for the trailer magic value
      #       as a end of the entry can be a possibility to continue.
      return False

    _, record_structure = self._BSM_TOKEN_TYPES.get(token_type, (u'', None))

    try:
      token = record_structure.parse_stream(file_object)
    except (IOError, construct.FieldError) as exception:
      parser_mediator.ProduceExtractionError((
          u'unable to parse BSM record at offset: 0x{0:08x} with error: '
          u'{1:s}.').format(record_start_offset, exception))
      return False

    event_type = bsmtoken.BSM_AUDIT_EVENT.get(
        token.bsm_header.event_type, u'UNKNOWN')
    event_type = u'{0:s} ({1:d})'.format(
        event_type, token.bsm_header.event_type)

    timestamp = (token.timestamp * 1000000) + token.microseconds
    date_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
        timestamp=timestamp)

    record_length = token.bsm_header.length
    record_end_offset = record_start_offset + record_length

    # A dict of tokens that has the entry.
    extra_tokens = {}

    # Read until we reach the end of the record.
    while file_object.tell() < record_end_offset:
      # Check if it is a known token.
      try:
        token_type = self._BSM_TOKEN.parse_stream(file_object)
      except (IOError, construct.FieldError):
        logging.warning(
            u'Unable to parse the Token ID at position: {0:d}'.format(
                file_object.tell()))
        return False

      _, record_structure = self._BSM_TOKEN_TYPES.get(token_type, (u'', None))

      if not record_structure:
        pending = record_end_offset - file_object.tell()
        new_extra_tokens = self.TryWithUntestedStructures(
            file_object, token_type, pending)
        extra_tokens.update(new_extra_tokens)
      else:
        token = record_structure.parse_stream(file_object)
        new_extra_tokens = self.FormatToken(token_type, token, file_object)
        extra_tokens.update(new_extra_tokens)

    if file_object.tell() > record_end_offset:
      logging.warning(
          u'Token ID {0:d} not expected at position 0x{1:08x}.'
          u'Jumping for the next entry.'.format(
              token_type, file_object.tell()))
      try:
        file_object.seek(
            record_end_offset - file_object.tell(), os.SEEK_CUR)
      except (IOError, construct.FieldError) as exception:
        logging.warning(
            u'Unable to jump to next entry with error: {0:s}'.format(exception))
        return False

    # BSM can be in more than one OS: BSD, Solaris and Mac OS X.
    if parser_mediator.platform != u'MacOSX':
      event_data = BSMEventData()
    else:
      event_data = MacBSMEventData()

      # In Mac OS X the last two tokens are the return status and the trailer.
      return_value = extra_tokens.get(u'BSM_TOKEN_RETURN32')
      if not return_value:
        return_value = extra_tokens.get(u'BSM_TOKEN_RETURN64')
      if not return_value:
        return_value = 'UNKNOWN'

      event_data.return_value = return_value

    event_data.event_type = event_type
    event_data.extra_tokens = extra_tokens
    event_data.offset = record_start_offset
    event_data.record_length = record_length

    # TODO: check why trailer was passed to event in original while
    # event was expecting record length.
    # if extra_tokens:
    #   trailer = extra_tokens.get(u'BSM_TOKEN_TRAILER', u'unknown')

    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_CREATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)

    return True

  def _RawToUTF8(self, byte_stream):
    """Copies a UTF-8 byte stream into a Unicode string.

    Args:
      byte_stream (bytes): byte stream containing an UTF-8 encoded string.

    Returns:
      str: A Unicode string.
    """
    try:
      string = byte_stream.decode(u'utf-8')
    except UnicodeDecodeError:
      logging.warning(
          u'Decode UTF8 failed, the message string may be cut short.')
      string = byte_stream.decode(u'utf-8', errors=u'ignore')
    return string.partition(b'\x00')[0]

  def ParseFileObject(self, parser_mediator, file_object, **kwargs):
    """Parses a BSM file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    try:
      is_bsm = self.VerifyFile(parser_mediator, file_object)
    except (IOError, construct.FieldError) as exception:
      raise errors.UnableToParseFile(
          u'Unable to parse BSM file with error: {0:s}'.format(exception))

    if not is_bsm:
      raise errors.UnableToParseFile(u'Not a BSM File, unable to parse.')

    file_object.seek(0, os.SEEK_SET)

    while self._ParseBSMEvent(parser_mediator, file_object):
      pass

  def VerifyFile(self, parser_mediator, file_object):
    """Check if the file is a BSM file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Returns:
      bool: True if this is a valid BSM file, False otherwise.
    """
    # First part of the entry is always a Header.
    try:
      token_type = self._BSM_TOKEN.parse_stream(file_object)
    except (IOError, construct.FieldError):
      return False

    if token_type not in self._BSM_HEADER_TOKEN_TYPES:
      return False

    _, record_structure = self._BSM_TOKEN_TYPES.get(token_type, (u'', None))

    try:
      header = record_structure.parse_stream(file_object)
    except (IOError, construct.FieldError):
      return False

    if header.bsm_header.version != self.AUDIT_HEADER_VERSION:
      return False

    try:
      token_identifier = self._BSM_TOKEN.parse_stream(file_object)
    except (IOError, construct.FieldError):
      return False

    # If is Mac OS X BSM file, next entry is a  text token indicating
    # if it is a normal start or it is a recovery track.
    if parser_mediator.platform == u'MacOSX':
      token_type, record_structure = self._BSM_TOKEN_TYPES.get(
          token_identifier, (u'', None))

      if not record_structure:
        return False

      if token_type != u'BSM_TOKEN_TEXT':
        logging.warning(u'It is not a valid first entry for Mac OS X BSM.')
        return False

      try:
        token = record_structure.parse_stream(file_object)
      except (IOError, construct.FieldError):
        return

      text = self._CopyUtf8ByteArrayToString(token.text)
      if (text != u'launchctl::Audit startup' and
          text != u'launchctl::Audit recovery'):
        logging.warning(u'It is not a valid first entry for Mac OS X BSM.')
        return False

    return True

  def TryWithUntestedStructures(self, file_object, token_id, pending):
    """Try to parse the pending part of the entry using untested structures.

    Args:
      file_object: BSM file.
      token_id: integer with the id that comes from the unknown token.
      pending: pending length of the entry.

    Returns:
      A list of extra tokens data that can be parsed using non-tested
      structures. A message indicating that a structure cannot be parsed
      is added for unparsed structures.
    """
    # Data from the unknown structure.
    start_position = file_object.tell()
    start_token_id = token_id
    extra_tokens = {}

    # Read all the "pending" bytes.
    try:
      if token_id in self._bsm_type_list_all:
        token = self._bsm_type_list_all[token_id][1].parse_stream(file_object)
        new_extra_tokens = self.FormatToken(token_id, token, file_object)
        extra_tokens.update(new_extra_tokens)
        while file_object.tell() < (start_position + pending):
          # Check if it is a known token.
          try:
            token_id = self._BSM_TOKEN.parse_stream(file_object)
          except (IOError, construct.FieldError):
            logging.warning(
                u'Unable to parse the Token ID at position: {0:d}'.format(
                    file_object.tell()))
            return
          if token_id not in self._bsm_type_list_all:
            break
          token = self._bsm_type_list_all[token_id][1].parse_stream(file_object)
          new_extra_tokens = self.FormatToken(token_id, token, file_object)
          extra_tokens.update(new_extra_tokens)
    except (IOError, construct.FieldError):
      token_id = 255

    next_entry = (start_position + pending)
    if file_object.tell() != next_entry:
      # Unknown Structure.
      logging.warning(u'Unknown Token at "0x{0:X}", ID: {1} (0x{2:X})'.format(
          start_position - 1, token_id, token_id))
      # TODO: another way to save this information must be found.
      extra_tokens.update(
          {u'message': self.MESSAGE_CAN_NOT_SAVE.format(
              start_position - 1, start_token_id)})
      # Move to next entry.
      file_object.seek(next_entry - file_object.tell(), os.SEEK_CUR)
      # It returns null list because it doesn't know witch structure was
      # the incorrect structure that makes that it can arrive to the spected
      # end of the entry.
      return {}
    return extra_tokens

  def FormatToken(self, token_id, token, file_object):
    """Parse the Token depending of the type of the structure.

    Args:
      token_id (int): identification of the token_type.
      token (structure): token struct to parse.
      file_object: BSM file.

    Returns:
      (dict): parsed Token values.

    Keys for returned dictionary are token name like BSM_TOKEN_SUBJECT32.
    Values of this dictionary are key-value pairs like terminal_ip:127.0.0.1.
    """
    if token_id not in self._bsm_type_list_all:
      return {}

    bsm_type, _ = self._bsm_type_list_all.get(token_id, [u'', u''])

    if token_id in self._BSM_UTF8_BYTE_ARRAY_TOKEN_TYPES:
      try:
        string = self._CopyUtf8ByteArrayToString(token.text)
      except TypeError:
        string = u'Unknown'
      return {bsm_type: string}

    elif token_id in self._BSM_RETURN_TOKEN_TYPES:
      return {bsm_type: {
          u'error': bsmtoken.BSM_ERRORS.get(token.status, u'Unknown'),
          u'token_status': token.status,
          u'call_status': token.return_value
      }}

    elif token_id in self._BSM_SUBJECT_TOKEN_TYPES:
      return {bsm_type: {
          u'aid': token.subject_data.audit_uid,
          u'euid': token.subject_data.effective_uid,
          u'egid': token.subject_data.effective_gid,
          u'uid': token.subject_data.real_uid,
          u'gid': token.subject_data.real_gid,
          u'pid': token.subject_data.pid,
          u'session_id': token.subject_data.session_id,
          u'terminal_port': token.terminal_port,
          u'terminal_ip': self._IPv4Format(token.ipv4)
      }}

    elif token_id in self._BSM_SUBJECT_EX_TOKEN_TYPES:
      if token.bsm_ip_type_short.net_type == self.AU_IPv6:
        ip = self._IPv6Format(
            token.bsm_ip_type_short.ip_addr.high,
            token.bsm_ip_type_short.ip_addr.low)
      elif token.bsm_ip_type_short.net_type == self.AU_IPv4:
        ip = self._IPv4Format(token.bsm_ip_type_short.ip_addr)
      else:
        ip = u'unknown'
      return {bsm_type: {
          u'aid': token.subject_data.audit_uid,
          u'euid': token.subject_data.effective_uid,
          u'egid': token.subject_data.effective_gid,
          u'uid': token.subject_data.real_uid,
          u'gid': token.subject_data.real_gid,
          u'pid': token.subject_data.pid,
          u'session_id': token.subject_data.session_id,
          u'terminal_port': token.terminal_port,
          u'terminal_ip': ip
      }}

    elif token_id in self._BSM_ARGUMENT_TOKEN_TYPES:
      string = self._CopyUtf8ByteArrayToString(token.text)
      return {bsm_type: {
          u'string': string,
          u'num_arg': token.num_arg,
          u'is': token.name_arg}}

    elif token_id in self._BSM_EXEV_TOKEN_TYPES:
      arguments = []
      for _ in range(0, token):
        sub_token = self.BSM_TOKEN_EXEC_ARGUMENT.parse_stream(file_object)
        string = self._CopyUtf8ByteArrayToString(sub_token.text)
        arguments.append(string)
      return {bsm_type: u' '.join(arguments)}

    elif bsm_type == u'BSM_TOKEN_AUT_SOCKINET32':
      return {bsm_type: {
          u'protocols':
          bsmtoken.BSM_PROTOCOLS.get(token.net_type, u'UNKNOWN'),
          u'net_type': token.net_type,
          u'port': token.port_number,
          u'address': self._IPv4Format(token.ipv4)
      }}

    elif bsm_type == u'BSM_TOKEN_AUT_SOCKINET128':
      return {bsm_type: {
          u'protocols':
          bsmtoken.BSM_PROTOCOLS.get(token.net_type, u'UNKNOWN'),
          u'net_type': token.net_type,
          u'port': token.port_number,
          u'address': self._IPv6Format(token.ipv6.high, token.ipv6.low)
      }}

    elif bsm_type == u'BSM_TOKEN_ADDR':
      return {bsm_type: self._IPv4Format(token)}

    elif bsm_type == u'BSM_TOKEN_IP':
      return {u'IPv4_Header': u'0x{0:s}]'.format(token.encode(u'hex'))}

    elif bsm_type == u'BSM_TOKEN_ADDR_EXT':
      return {bsm_type: {
          u'protocols':
          bsmtoken.BSM_PROTOCOLS.get(token.net_type, u'UNKNOWN'),
          u'net_type': token.net_type,
          u'address': self._IPv6Format(token.ipv6.high, token.ipv6.low)
      }}

    elif bsm_type == u'BSM_TOKEN_PORT':
      return {bsm_type: token}

    elif bsm_type == u'BSM_TOKEN_TRAILER':
      return {bsm_type: token.record_length}

    elif bsm_type == u'BSM_TOKEN_FILE':
      # TODO: if this timestamp is usefull, it must be extracted as a separate
      #       event object.
      timestamp = timelib.Timestamp.FromPosixTimeWithMicrosecond(
          token.timestamp, token.microseconds)
      date_time = timelib.Timestamp.CopyToDatetime(timestamp, pytz.UTC)
      date_time_string = date_time.strftime(u'%Y-%m-%d %H:%M:%S')

      string = self._CopyUtf8ByteArrayToString(token.text)
      return {bsm_type: {u'string': string, u'timestamp': date_time_string}}

    elif bsm_type == u'BSM_TOKEN_IPC':
      return {bsm_type: {
          u'object_type': token.object_type,
          u'object_id': token.object_id
      }}

    elif token_id in self._BSM_PROCESS_TOKEN_TYPES:
      return {bsm_type: {
          u'aid': token.subject_data.audit_uid,
          u'euid': token.subject_data.effective_uid,
          u'egid': token.subject_data.effective_gid,
          u'uid': token.subject_data.real_uid,
          u'gid': token.subject_data.real_gid,
          u'pid': token.subject_data.pid,
          u'session_id': token.subject_data.session_id,
          u'terminal_port': token.terminal_port,
          u'terminal_ip': self._IPv4Format(token.ipv4)
      }}

    elif token_id in self._BSM_PROCESS_EX_TOKEN_TYPES:
      if token.bsm_ip_type_short.net_type == self.AU_IPv6:
        ip = self._IPv6Format(
            token.bsm_ip_type_short.ip_addr.high,
            token.bsm_ip_type_short.ip_addr.low)
      elif token.bsm_ip_type_short.net_type == self.AU_IPv4:
        ip = self._IPv4Format(token.bsm_ip_type_short.ip_addr)
      else:
        ip = u'unknown'
      return {bsm_type: {
          u'aid': token.subject_data.audit_uid,
          u'euid': token.subject_data.effective_uid,
          u'egid': token.subject_data.effective_gid,
          u'uid': token.subject_data.real_uid,
          u'gid': token.subject_data.real_gid,
          u'pid': token.subject_data.pid,
          u'session_id': token.subject_data.session_id,
          u'terminal_port': token.terminal_port,
          u'terminal_ip': ip
      }}

    elif bsm_type == u'BSM_TOKEN_DATA':
      data = []
      data_type = bsmtoken.BSM_TOKEN_DATA_TYPE.get(token.data_type, u'')

      if data_type == u'AUR_CHAR':
        for _ in range(token.unit_count):
          data.append(self.BSM_TOKEN_DATA_CHAR.parse_stream(file_object))

      elif data_type == u'AUR_SHORT':
        for _ in range(token.unit_count):
          data.append(self.BSM_TOKEN_DATA_SHORT.parse_stream(file_object))

      elif data_type == u'AUR_INT32':
        for _ in range(token.unit_count):
          data.append(self.BSM_TOKEN_DATA_INTEGER.parse_stream(file_object))

      else:
        data.append(u'Unknown type data')

      # TODO: the data when it is string ends with ".", HW a space is return
      #       after uses the UTF-8 conversion.
      return {bsm_type: {
          u'format': bsmtoken.BSM_TOKEN_DATA_PRINT[token.how_to_print],
          u'data':
          u'{0}'.format(self._RawToUTF8(''.join(map(str, data))))
      }}

    elif token_id in self._BSM_ATTR_TOKEN_TYPES:
      return {bsm_type: {
          u'mode': token.file_mode,
          u'uid': token.uid,
          u'gid': token.gid,
          u'system_id': token.file_system_id,
          u'node_id': token.file_system_node_id,
          u'device': token.device}}

    elif bsm_type == u'BSM_TOKEN_GROUPS':
      arguments = []
      for _ in range(token):
        arguments.append(
            self._RawToUTF8(
                self.BSM_TOKEN_DATA_INTEGER.parse_stream(file_object)))
      return {bsm_type: u','.join(arguments)}

    elif bsm_type == u'BSM_TOKEN_AUT_SOCKINET32_EX':
      if bsmtoken.BSM_PROTOCOLS.get(token.socket_domain, u'') == u'INET6':
        saddr = self._IPv6Format(
            token.structure_addr_port.saddr_high,
            token.structure_addr_port.saddr_low)
        daddr = self._IPv6Format(
            token.structure_addr_port.daddr_high,
            token.structure_addr_port.daddr_low)
      else:
        saddr = self._IPv4Format(token.structure_addr_port.source_address)
        daddr = self._IPv4Format(token.structure_addr_port.destination_address)

      return {bsm_type:{
          u'from': saddr,
          u'from_port': token.structure_addr_port.source_port,
          u'to': daddr,
          u'to_port': token.structure_addr_port.destination_port}}

    elif bsm_type == u'BSM_TOKEN_IPC_PERM':
      return {bsm_type: {
          u'user_id': token.user_id,
          u'group_id': token.group_id,
          u'creator_user_id': token.creator_user_id,
          u'creator_group_id': token.creator_group_id,
          u'access': token.access_mode}}

    elif bsm_type == u'BSM_TOKEN_SOCKET_UNIX':
      string = self._CopyUtf8ByteArrayToString(token.path)
      return {bsm_type: {u'family': token.family, u'path': string}}

    elif bsm_type == u'BSM_TOKEN_OPAQUE':
      string = self._CopyByteArrayToBase16String(token.text)
      return {bsm_type: string}

    elif bsm_type == u'BSM_TOKEN_SEQUENCE':
      return {bsm_type: token}


manager.ParsersManager.RegisterParser(BSMParser)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the timemachine plist plugin."""

import unittest

from plaso.formatters import plist  # pylint: disable=unused-import
from plaso.parsers.plist_plugins import timemachine

from tests import test_lib as shared_test_lib
from tests.parsers.plist_plugins import test_lib


class TimeMachinePluginTest(test_lib.PlistPluginTestCase):
  """Tests for the timemachine plist plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'com.apple.TimeMachine.plist'])
  def testProcess(self):
    """Tests the Process function."""
    plist_name = u'com.apple.TimeMachine.plist'

    plugin = timemachine.TimeMachinePlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    self.assertEqual(storage_writer.number_of_events, 13)

    # The order in which PlistParser generates events is nondeterministic
    # hence we sort the events.
    events = list(storage_writer.GetSortedEvents())

    expected_timestamps = [
        1379165051000000, 1380098455000000, 1380810276000000, 1381883538000000,
        1382647890000000, 1383351739000000, 1384090020000000, 1385130914000000,
        1386265911000000, 1386689852000000, 1387723091000000, 1388840950000000,
        1388842718000000]
    timestamps = sorted([event.timestamp for event in events])

    self.assertEqual(timestamps, expected_timestamps)

    event = events[0]
    self.assertEqual(event.root, u'/Destinations')
    self.assertEqual(event.key, u'item/SnapshotDates')

    expected_description = (
        u'TimeMachine Backup in BackUpFast '
        u'(5B33C22B-A4A1-4024-A2F5-C9979C4AAAAA)')
    self.assertEqual(event.desc, expected_description)

    expected_message = u'/Destinations/item/SnapshotDates {0:s}'.format(
        expected_description)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])
    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the CLI argument helper manager."""

import argparse
import unittest

from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.lib import errors

from tests.cli.helpers import test_lib


class AnotherTestHelper(interface.ArgumentsHelper):
  """Another test CLI argument helper."""

  NAME = 'another_test_helper'
  DESCRIPTION = u'Another test helper that does nothing.'

  @classmethod
  def AddArguments(cls, argument_group):
    """Add command line arguments to an argument group."""
    argument_group.add_argument(
        u'-c', u'--correcto', dest=u'correcto', action='store_true',
        default=False, help=u'The correcto option.')

  @classmethod
  def ParseOptions(cls, options, unused_config_object):
    """Parse and validate the configurational options."""
    if not hasattr(options, 'correcto'):
      raise errors.BadConfigOption(u'Correcto not set.')

    if not isinstance(getattr(options, u'correcto', None), bool):
      raise errors.BadConfigOption(u'Correcto wrongly formatted.')


class HelperManagerTest(unittest.TestCase):
  """Tests the parsers manager."""

  def testHelperRegistration(self):
    """Tests the RegisterHelper and DeregisterHelper functions."""
    # pylint: disable=protected-access
    number_of_helpers = len(manager.ArgumentHelperManager._helper_classes)

    manager.ArgumentHelperManager.RegisterHelper(test_lib.TestHelper)
    self.assertEqual(
        len(manager.ArgumentHelperManager._helper_classes),
        number_of_helpers + 1)

    with self.assertRaises(KeyError):
      manager.ArgumentHelperManager.RegisterHelper(test_lib.TestHelper)

    manager.ArgumentHelperManager.DeregisterHelper(test_lib.TestHelper)
    self.assertEqual(
        len(manager.ArgumentHelperManager._helper_classes),
        number_of_helpers)

  def testCommandLineArguments(self):
    """Test the AddCommandLineArguments and function."""
    manager.ArgumentHelperManager.RegisterHelpers([
        test_lib.TestHelper, AnotherTestHelper])

    arg_parser = argparse.ArgumentParser(conflict_handler=u'resolve')
    manager.ArgumentHelperManager.AddCommandLineArguments(arg_parser)

    # Assert the parameters have been set.
    options = arg_parser.parse_args([])

    self.assertTrue(hasattr(options, u'dynamic'))
    self.assertTrue(hasattr(options, u'correcto'))

    self.assertFalse(hasattr(options, u'foobar'))

    # Make the parameters fail validation.
    options.correcto = 'sfd'
    with self.assertRaises(errors.BadConfigOption):
      manager.ArgumentHelperManager.ParseOptions(options, None)

    options.correcto = True
    options.dynamic = 'now stuff'
    manager.ArgumentHelperManager.ParseOptions(options, None)

    manager.ArgumentHelperManager.DeregisterHelper(test_lib.TestHelper)
    manager.ArgumentHelperManager.DeregisterHelper(AnotherTestHelper)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""A parser for Portable Executable format files."""

import pefile

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.lib import specification
from plaso.parsers import interface
from plaso.parsers import manager


class PEEventData(events.EventData):
  """Portable Executable (PE) event data.

  Attributes:
    dll_name (str): name of an imported DLL.
    imphash (str): "Import Hash" of the pe file the event relates to. Also see:
        https://www.mandiant.com/blog/tracking-malware-import-hashing
    pe_type (str): type of PE file the event relates to.
    section_names (list[str]): names of the PE file's sections.
  """

  DATA_TYPE = u'pe'

  def __init__(self):
    """Initializes event data."""
    super(PEEventData, self).__init__(data_type=self.DATA_TYPE)
    self.dll_name = None
    self.imphash = None
    self.pe_type = None
    self.section_names = None


class PEParser(interface.FileObjectParser):
  """Parser for Portable Executable (PE) files."""

  NAME = u'pe'
  DESCRIPTION = u'Parser for Portable Executable (PE) files.'

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification."""
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(b'MZ', offset=0)
    return format_specification

  def _GetSectionNames(self, pefile_object):
    """Retrieves all PE section names.

    Args:
      pefile_object (pefile.PE): pefile object.

    Returns:
      list[str]: names of the sections.
    """
    section_names = []
    for section in pefile_object.sections:
      section_name = getattr(section, u'Name', b'')
      # Ensure the name is decoded correctly.
      try:
        section_name = u'{0:s}'.format(section_name.decode(u'unicode_escape'))
      except UnicodeDecodeError:
        section_name = u'{0:s}'.format(repr(section_name))
      section_names.append(section_name)

    return section_names

  def _GetImportTimestamps(self, pefile_object):
    """Retrieves timestamps from the import directory, if available.

    Args:
      pefile_object (pefile.PE): pefile object.

    Returns:
      list[int]: import timestamps.
    """
    import_timestamps = []
    if not hasattr(pefile_object, u'DIRECTORY_ENTRY_IMPORT'):
      return import_timestamps
    for importdata in pefile_object.DIRECTORY_ENTRY_IMPORT:
      timestamp = getattr(importdata.struct, u'TimeDateStamp', 0)
      dll_name = getattr(importdata, u'dll', u'<NO DLL NAME>')
      if timestamp:
        import_timestamps.append([dll_name, timestamp])
    return import_timestamps

  def _GetResourceTimestamps(self, pefile_object):
    """Retrieves timestamps from resource directory entries, if available.

    Args:
      pefile_object (pefile.PE): pefile object.

    Returns:
      list[int]: resource timestamps.
    """
    timestamps = []
    if not hasattr(pefile_object, u'DIRECTORY_ENTRY_RESOURCE'):
      return timestamps
    for entrydata in pefile_object.DIRECTORY_ENTRY_RESOURCE.entries:
      directory = entrydata.directory
      timestamp = getattr(directory, u'TimeDateStamp', 0)
      if timestamp:
        timestamps.append(timestamp)
    return timestamps

  def _GetLoadConfigTimestamp(self, pefile_object):
    """Retrieves the timestamp from the Load Configuration directory.

    Args:
      pefile_object (pefile.PE): pefile object.

    Returns:
      int: load configuration timestamps.
    """
    if not hasattr(pefile_object, u'DIRECTORY_ENTRY_LOAD_CONFIG'):
      return
    timestamp = getattr(
        pefile_object.DIRECTORY_ENTRY_LOAD_CONFIG.struct, u'TimeDateStamp', 0)
    return timestamp

  def _GetDelayImportTimestamps(self, pefile_object):
    """Retrieves timestamps from delay import entries, if available.

    Args:
      pefile_object (pefile.PE): pefile object.

    Returns:
      A list two-element lists, where the first element is the name of the DLL
      being imported, and the second is the timestamp of the entry.
    """
    delay_import_timestamps = []
    if not hasattr(pefile_object, u'DIRECTORY_ENTRY_DELAY_IMPORT'):
      return delay_import_timestamps
    for importdata in pefile_object.DIRECTORY_ENTRY_DELAY_IMPORT:
      dll_name = importdata.dll
      timestamp = getattr(importdata.struct, u'dwTimeStamp', 0)
      delay_import_timestamps.append([dll_name, timestamp])
    return delay_import_timestamps

  def _GetPEType(self, pefile_object):
    """Retrieves the type of the PE file.

    Args:
      pefile_object (pefile.PE): pefile object.

    Returns:
      str: type of the Portable Executable (PE) file.
    """
    if pefile_object.is_dll():
      return u'Dynamic Link Library (DLL)'
    if pefile_object.is_exe():
      return u'Executable (EXE)'
    if pefile_object.is_driver():
      return u'Driver (SYS)'
    return u'Unknown PE type'

  def ParseFileObject(self, parser_mediator, file_object, **unused_kwargs):
    """Parses a Portable Executable (PE) file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): a file-like object.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    pe_data = file_object.read()
    try:
      pefile_object = pefile.PE(data=pe_data, fast_load=True)
      pefile_object.parse_data_directories(
          directories=[
              pefile.DIRECTORY_ENTRY[u'IMAGE_DIRECTORY_ENTRY_IMPORT'],
              pefile.DIRECTORY_ENTRY[u'IMAGE_DIRECTORY_ENTRY_EXPORT'],
              pefile.DIRECTORY_ENTRY[u'IMAGE_DIRECTORY_ENTRY_RESOURCE'],
              pefile.DIRECTORY_ENTRY[u'IMAGE_DIRECTORY_ENTRY_DELAY_IMPORT'],])
    except:
      raise errors.UnableToParseFile()

    event_data = PEEventData()
    event_data.imphash = pefile_object.get_imphash()
    event_data.pe_type = self._GetPEType(pefile_object)
    event_data.section_names = self._GetSectionNames(pefile_object)

    # TODO: remove after refactoring the pe event formatter.
    event_data.data_type = u'pe:compilation:compilation_time'

    timestamp = getattr(pefile_object.FILE_HEADER, u'TimeDateStamp', None)
    # TODO: handle timestamp is None.
    date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_CREATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)

    for dll_name, timestamp in self._GetImportTimestamps(pefile_object):
      if timestamp:
        event_data.dll_name = dll_name
        event_data.data_type = u'pe:import:import_time'

        date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
        parser_mediator.ProduceEventWithEventData(event, event_data)

    for dll_name, timestamp in self._GetDelayImportTimestamps(pefile_object):
      if timestamp:
        event_data.dll_name = dll_name
        event_data.data_type = u'pe:delay_import:import_time'

        date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
        parser_mediator.ProduceEventWithEventData(event, event_data)

    event_data.dll_name = None

    for timestamp in self._GetResourceTimestamps(pefile_object):
      if timestamp:
        event_data.data_type = u'pe:resource:creation_time'

        date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
        parser_mediator.ProduceEventWithEventData(event, event_data)

    timestamp = self._GetLoadConfigTimestamp(pefile_object)
    if timestamp:
      event_data.data_type = u'pe:load_config:modification_time'

      date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)


manager.ParsersManager.RegisterParser(PEParser)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Windows Shortcut (LNK) event formatter."""

import unittest

from plaso.formatters import winlnk

from tests.formatters import test_lib


class WinLnkLinkFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Windows Shortcut (LNK) event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = winlnk.WinLnkLinkFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = winlnk.WinLnkLinkFormatter()

    expected_attribute_names = [
        u'description', u'file_size', u'file_attribute_flags', u'drive_type',
        u'drive_serial_number', u'volume_label', u'local_path',
        u'network_path', u'command_line_arguments', u'env_var_location',
        u'relative_path', u'working_directory', u'icon_location',
        u'link_target']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*_ coding: utf-8 -*-
"""Tests for the SCCM Logs Parser."""

import unittest

from plaso.formatters import sccm  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import sccm

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class SCCMLogsUnitTest(test_lib.ParserTestCase):
  """Tests for the SCCM Logs Parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'sccm_various.log'])
  def testParse(self):
    """Tests for the Parse function."""
    parser = sccm.SCCMParser()
    storage_writer = self._ParseFile([u'sccm_various.log'], parser)

    self.assertEqual(storage_writer.number_of_events, 10)

    events = list(storage_writer.GetEvents())

    event = events[0]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-11-28 14:03:19.766')
    self.assertEqual(event.timestamp, expected_timestamp)

    # Test timestamps with seven digits after seconds.
    event = events[3]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2015-01-02 10:22:50.873496')
    self.assertEqual(event.timestamp, expected_timestamp)

    # Test timestamps with '-' in microseconds.
    event = events[7]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-12-28 07:59:43.373')
    self.assertEqual(event.timestamp, expected_timestamp)

    # Test timestamps with '+' in microseconds.
    event = events[9]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-11-24 09:52:13.827')
    self.assertEqual(event.timestamp, expected_timestamp)

    # Test full and short message formats.
    event = events[4]
    expected_message = (
        u'ContentAccess Releasing content request '
        u'{4EA97AD6-E7E2-4583-92B9-21F532501337}')

    expected_short_message = (
        u'Releasing content request '
        u'{4EA97AD6-E7E2-4583-92B9-21F532501337}')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for Java Cache IDX files."""

# TODO:
#  * 6.02 files did not retain IP addresses. However, the
#    deploy_resource_codebase header field may contain the host IP.
#    This needs to be researched further, as that field may not always
#    be present. 6.02 files will currently return 'Unknown'.

import os

import construct

from dfdatetime import java_time as dfdatetime_java_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import interface
from plaso.parsers import manager


class JavaIDXEventData(events.EventData):
  """Java IDX cache file event data.

  Attributes:
    idx_version (str): format version of IDX file.
    ip_address (str): IP address of the host in the URL.
    url (str): URL of the downloaded file.
  """

  DATA_TYPE = u'java:download:idx'

  def __init__(self):
    """Initializes event data."""
    super(JavaIDXEventData, self).__init__(data_type=self.DATA_TYPE)
    self.idx_version = None
    self.ip_address = None
    self.url = None


class JavaIDXParser(interface.FileObjectParser):
  """Parse Java WebStart Cache IDX files for download events.

  There are five structures defined. 6.02 files had one generic section
  that retained all data. From 6.03, the file went to a multi-section
  format where later sections were optional and had variable-lengths.
  6.03, 6.04, and 6.05 files all have their main data section (#2)
  begin at offset 128. The short structure is because 6.05 files
  deviate after the 8th byte. So, grab the first 8 bytes to ensure it's
  valid, get the file version, then continue on with the correct
  structures.
  """

  _INITIAL_FILE_OFFSET = None

  NAME = u'java_idx'
  DESCRIPTION = u'Parser for Java WebStart Cache IDX files.'

  IDX_SHORT_STRUCT = construct.Struct(
      u'magic',
      construct.UBInt8(u'busy'),
      construct.UBInt8(u'incomplete'),
      construct.UBInt32(u'idx_version'))

  IDX_602_STRUCT = construct.Struct(
      u'IDX_602_Full',
      construct.UBInt16(u'null_space'),
      construct.UBInt8(u'shortcut'),
      construct.UBInt32(u'content_length'),
      construct.UBInt64(u'last_modified_date'),
      construct.UBInt64(u'expiration_date'),
      construct.PascalString(
          u'version_string', length_field=construct.UBInt16(u'length')),
      construct.PascalString(
          u'url', length_field=construct.UBInt16(u'length')),
      construct.PascalString(
          u'namespace', length_field=construct.UBInt16(u'length')),
      construct.UBInt32(u'FieldCount'))

  IDX_605_SECTION_ONE_STRUCT = construct.Struct(
      u'IDX_605_Section1',
      construct.UBInt8(u'shortcut'),
      construct.UBInt32(u'content_length'),
      construct.UBInt64(u'last_modified_date'),
      construct.UBInt64(u'expiration_date'),
      construct.UBInt64(u'validation_date'),
      construct.UBInt8(u'signed'),
      construct.UBInt32(u'sec2len'),
      construct.UBInt32(u'sec3len'),
      construct.UBInt32(u'sec4len'))

  IDX_605_SECTION_TWO_STRUCT = construct.Struct(
      u'IDX_605_Section2',
      construct.PascalString(
          u'version', length_field=construct.UBInt16(u'length')),
      construct.PascalString(
          u'url', length_field=construct.UBInt16(u'length')),
      construct.PascalString(
          u'namespec', length_field=construct.UBInt16(u'length')),
      construct.PascalString(
          u'ip_address', length_field=construct.UBInt16(u'length')),
      construct.UBInt32(u'FieldCount'))

  # Java uses Pascal-style strings, but with a 2-byte length field.
  JAVA_READUTF_STRING = construct.Struct(
      u'Java.ReadUTF',
      construct.PascalString(
          u'string', length_field=construct.UBInt16(u'length')))

  def ParseFileObject(self, parser_mediator, file_object, **kwargs):
    """Parses a Java WebStart Cache IDX file-like object.

    Args:
      parser_mediator: A parser mediator object (instance of ParserMediator).
      file_object: A file-like object.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    file_object.seek(0, os.SEEK_SET)
    try:
      magic = self.IDX_SHORT_STRUCT.parse_stream(file_object)
    except (IOError, construct.FieldError) as exception:
      raise errors.UnableToParseFile(
          u'Unable to parse Java IDX file with error: {0:s}.'.format(exception))

    # Fields magic.busy and magic.incomplete are normally 0x00. They
    # are set to 0x01 if the file is currently being downloaded. Logic
    # checks for > 1 to avoid a race condition and still reject any
    # file with other data.
    # Field magic.idx_version is the file version, of which only
    # certain versions are supported.
    if magic.busy > 1 or magic.incomplete > 1:
      raise errors.UnableToParseFile(u'Not a valid Java IDX file')

    if not magic.idx_version in [602, 603, 604, 605]:
      raise errors.UnableToParseFile(u'Not a valid Java IDX file')

    # Obtain the relevant values from the file. The last modified date
    # denotes when the file was last modified on the HOST. For example,
    # when the file was uploaded to a web server.
    if magic.idx_version == 602:
      section_one = self.IDX_602_STRUCT.parse_stream(file_object)
      last_modified_date = section_one.last_modified_date
      url = section_one.url
      ip_address = u'Unknown'
      http_header_count = section_one.FieldCount
    elif magic.idx_version in [603, 604, 605]:

      # IDX 6.03 and 6.04 have two unused bytes before the structure.
      if magic.idx_version in [603, 604]:
        file_object.read(2)

      # IDX 6.03, 6.04, and 6.05 files use the same structures for the
      # remaining data.
      section_one = self.IDX_605_SECTION_ONE_STRUCT.parse_stream(file_object)
      last_modified_date = section_one.last_modified_date
      if file_object.get_size() > 128:
        file_object.seek(128, os.SEEK_SET)  # Static offset for section 2.
        section_two = self.IDX_605_SECTION_TWO_STRUCT.parse_stream(file_object)
        url = section_two.url
        ip_address = section_two.ip_address
        http_header_count = section_two.FieldCount
      else:
        url = u'Unknown'
        ip_address = u'Unknown'
        http_header_count = 0

    # File offset is now just prior to HTTP headers. Make sure there
    # are headers, and then parse them to retrieve the download date.
    download_date = None
    for field in range(0, http_header_count):
      field = self.JAVA_READUTF_STRING.parse_stream(file_object)
      value = self.JAVA_READUTF_STRING.parse_stream(file_object)
      if field.string == u'date':
        # Time string "should" be in UTC or have an associated time zone
        # information in the string itself. If that is not the case then
        # there is no reliable method for plaso to determine the proper
        # timezone, so the assumption is that it is UTC.
        try:
          download_date = timelib.Timestamp.FromTimeString(
              value.string, gmt_as_timezone=False)
        except errors.TimestampError:
          download_date = None
          parser_mediator.ProduceExtractionError(
              u'Unable to parse time value: {0:s}'.format(value.string))

    if not url or not ip_address:
      raise errors.UnableToParseFile(
          u'Unexpected Error: URL or IP address not found in file.')

    event_data = JavaIDXEventData()
    event_data.idx_version = magic.idx_version
    event_data.ip_address = ip_address
    event_data.url = url

    date_time = dfdatetime_java_time.JavaTime(timestamp=last_modified_date)
    # TODO: Move the timestamp description into eventdata.
    event = time_events.DateTimeValuesEvent(date_time, u'File Hosted Date')
    parser_mediator.ProduceEventWithEventData(event, event_data)

    if section_one:
      expiration_date = section_one.get(u'expiration_date', None)
      if expiration_date:
        date_time = dfdatetime_java_time.JavaTime(
            timestamp=expiration_date)
        event = time_events.DateTimeValuesEvent(
            date_time, definitions.TIME_DESCRIPTION_EXPIRATION)
        parser_mediator.ProduceEventWithEventData(event, event_data)

    if download_date:
      event = time_events.TimestampEvent(
          download_date, definitions.TIME_DESCRIPTION_FILE_DOWNLOADED)
      parser_mediator.ProduceEventWithEventData(event, event_data)


manager.ParsersManager.RegisterParser(JavaIDXParser)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Windows preprocess plug-ins."""

import unittest

from dfvfs.helpers import fake_file_system_builder
from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.preprocessors import windows

from tests import test_lib as shared_test_lib
from tests.preprocessors import test_lib


class WindowsCodepagePlugin(test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the Windows codepage plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'SYSTEM'])
  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    plugin = windows.WindowsCodepagePlugin()
    knowledge_base = self._RunPreprocessorPluginOnWindowsRegistryValueSystem(
        plugin)

    self.assertEqual(knowledge_base.codepage, u'cp1252')


class WindowsHostnamePluginTest(test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the Windows hostname plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'SYSTEM'])
  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    plugin = windows.WindowsHostnamePlugin()
    knowledge_base = self._RunPreprocessorPluginOnWindowsRegistryValueSystem(
        plugin)

    self.assertEqual(knowledge_base.hostname, u'WKS-WIN732BITA')


class WindowsProgramFilesEnvironmentVariablePluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the %ProgramFiles% environment variable plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'SOFTWARE'])
  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    plugin = (
        windows.WindowsProgramFilesEnvironmentVariablePlugin())
    knowledge_base = self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        plugin)

    environment_variable = knowledge_base.GetEnvironmentVariable(
        u'ProgramFiles')
    self.assertIsNotNone(environment_variable)
    self.assertEqual(environment_variable.value, u'C:\\Program Files')


class WindowsProgramFilesX86EnvironmentVariablePluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the %ProgramFilesX86% environment variable plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'SOFTWARE'])
  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    plugin = (
        windows.WindowsProgramFilesX86EnvironmentVariablePlugin())
    knowledge_base = self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        plugin)

    environment_variable = knowledge_base.GetEnvironmentVariable(
        u'ProgramFilesX86')
    # The test SOFTWARE Registry file does not contain a value for
    # the Program Files X86 path.
    self.assertIsNone(environment_variable)


class WindowsSystemRootEnvironmentVariablePluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the %SystemRoot% environment variable plugin."""

  _FILE_DATA = b'regf'

  def testParsePathSpecification(self):
    """Tests the _ParsePathSpecification function."""
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile(
        u'/Windows/System32/config/SYSTEM', self._FILE_DATA)

    mount_point = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_FAKE, location=u'/')

    plugin = (
        windows.WindowsSystemRootEnvironmentVariablePlugin())
    knowledge_base = self._RunPreprocessorPluginOnFileSystem(
        file_system_builder.file_system, mount_point, plugin)

    environment_variable = knowledge_base.GetEnvironmentVariable(u'SystemRoot')
    self.assertIsNotNone(environment_variable)
    self.assertEqual(environment_variable.value, u'\\Windows')


class WindowsSystemProductPluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the system product information plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'SOFTWARE'])
  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    plugin = windows.WindowsSystemProductPlugin()
    knowledge_base = self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        plugin)

    osversion = knowledge_base.GetValue(u'operating_system_product')
    self.assertEqual(osversion, u'Windows 7 Ultimate')


class WindowsSystemVersionPluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the system version information plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'SOFTWARE'])
  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    plugin = windows.WindowsSystemVersionPlugin()
    knowledge_base = self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        plugin)

    osversion = knowledge_base.GetValue(u'operating_system_version')
    self.assertEqual(osversion, u'6.1')


class WindowsTimeZonePluginTest(test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the time zone plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'SYSTEM'])
  def testParseValueData(self):
    """Tests the _ParseValueData function."""
    plugin = windows.WindowsTimeZonePlugin()
    knowledge_base = self._RunPreprocessorPluginOnWindowsRegistryValueSystem(
        plugin)

    self.assertEqual(knowledge_base.timezone.zone, u'EST5EDT')


class WindowsUserAccountsPluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the Windows user accounts artifacts mapping."""

  # pylint: disable=protected-access

  @shared_test_lib.skipUnlessHasTestFile([u'SOFTWARE'])
  def testParseKey(self):
    """Tests the _ParseKey function."""
    plugin = windows.WindowsUserAccountsPlugin()
    knowledge_base = self._RunPreprocessorPluginOnWindowsRegistryValueSoftware(
        plugin)

    users = sorted(
        knowledge_base.user_accounts,
        key=lambda user_account: user_account.identifier)
    self.assertIsNotNone(users)
    self.assertEqual(len(users), 11)

    user_account = users[9]

    expected_sid = u'S-1-5-21-2036804247-3058324640-2116585241-1114'
    self.assertEqual(user_account.identifier, expected_sid)
    self.assertEqual(user_account.username, u'rsydow')
    self.assertEqual(user_account.user_directory, u'C:\\Users\\rsydow')


class WindowsWinDirEnvironmentVariablePluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the %WinDir% environment variable plugin."""

  _FILE_DATA = b'regf'

  def testParsePathSpecification(self):
    """Tests the _ParsePathSpecification function."""
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile(
        u'/Windows/System32/config/SYSTEM', self._FILE_DATA)

    mount_point = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_FAKE, location=u'/')

    plugin = (
        windows.WindowsWinDirEnvironmentVariablePlugin())
    knowledge_base = self._RunPreprocessorPluginOnFileSystem(
        file_system_builder.file_system, mount_point, plugin)

    environment_variable = knowledge_base.GetEnvironmentVariable(u'WinDir')
    self.assertIsNotNone(environment_variable)
    self.assertEqual(environment_variable.value, u'\\Windows')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The extraction CLI tool."""

# The following import makes sure the analyzers are registered.
from plaso import analyzers  # pylint: disable=unused-import

# The following import makes sure the parsers are registered.
from plaso import parsers  # pylint: disable=unused-import

from plaso.cli import storage_media_tool
from plaso.lib import errors


class ExtractionTool(storage_media_tool.StorageMediaTool):
  """Extraction CLI tool."""

  # Approximately 250 MB of queued items per worker.
  _DEFAULT_QUEUE_SIZE = 125000

  _BYTES_IN_A_MIB = 1024 * 1024

  def __init__(self, input_reader=None, output_writer=None):
    """Initializes an CLI tool.

    Args:
      input_reader (Optional[InputReader]): input reader, where None indicates
          that the stdin input reader should be used.
      output_writer (Optional[OutputWriter]): output writer, where None
          indicates that the stdout output writer should be used.
    """
    super(ExtractionTool, self).__init__(
        input_reader=input_reader, output_writer=output_writer)
    self._artifacts_registry = None
    self._buffer_size = 0
    self._force_preprocessing = False
    self._mount_path = None
    self._operating_system = None
    self._preferred_year = None
    self._process_archives = False
    self._process_compressed_streams = True
    self._queue_size = self._DEFAULT_QUEUE_SIZE
    self._single_process_mode = False

  def _ParsePerformanceOptions(self, options):
    """Parses the performance options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    self._buffer_size = getattr(options, u'buffer_size', 0)
    if self._buffer_size:
      # TODO: turn this into a generic function that supports more size
      # suffixes both MB and MiB and also that does not allow m as a valid
      # indicator for MiB since m represents milli not Mega.
      try:
        if self._buffer_size[-1].lower() == u'm':
          self._buffer_size = int(self._buffer_size[:-1], 10)
          self._buffer_size *= self._BYTES_IN_A_MIB
        else:
          self._buffer_size = int(self._buffer_size, 10)
      except ValueError:
        raise errors.BadConfigOption(
            u'Invalid buffer size: {0:s}.'.format(self._buffer_size))

    self._queue_size = self.ParseNumericOption(options, u'queue_size')

  def AddPerformanceOptions(self, argument_group):
    """Adds the performance options to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        u'--buffer_size', u'--buffer-size', u'--bs', dest=u'buffer_size',
        action=u'store', default=0, help=(
            u'The buffer size for the output (defaults to 196MiB).'))

    argument_group.add_argument(
        u'--queue_size', u'--queue-size', dest=u'queue_size', action=u'store',
        default=0, help=(
            u'The maximum number of queued items per worker '
            u'(defaults to {0:d})').format(self._DEFAULT_QUEUE_SIZE))



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""This file contains a unit test for ASL securityd log parser."""

import unittest

from plaso.formatters import mac_securityd  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import mac_securityd

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class MacSecurityUnitTest(test_lib.ParserTestCase):
  """A unit test for the ASL securityd log parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'security.log'])
  def testParseFile(self):
    """Test parsing of a ASL securityd log file."""
    parser = mac_securityd.MacSecuritydLogParser()
    knowledge_base_values = {u'year': 2013}
    storage_writer = self._ParseFile(
        [u'security.log'], parser,
        knowledge_base_values=knowledge_base_values)

    self.assertEqual(storage_writer.number_of_events, 9)

    events = list(storage_writer.GetEvents())

    event = events[0]
    expected_message = (
        u'Sender: secd (1) Level: Error Facility: user '
        u'Text: securityd_xpc_dictionary_handler EscrowSecurityAl'
        u'[3273] DeviceInCircle \xdeetta \xe6tti a\xf0 '
        u'virka l\xedka, setja \xedslensku inn.')
    expected_short_message = (
        u'Text: securityd_xpc_dictionary_handler '
        u'EscrowSecurityAl[3273] DeviceInCircle ...')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-02-26 19:11:56')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.sender, u'secd')
    self.assertEqual(event.sender_pid, 1)
    self.assertEqual(event.facility, u'user')
    self.assertEqual(event.security_api, u'unknown')
    self.assertEqual(event.caller, u'unknown')
    self.assertEqual(event.level, u'Error')
    expected_message = (
        u'securityd_xpc_dictionary_handler EscrowSecurityAl'
        u'[3273] DeviceInCircle \xdeetta \xe6tti a\xf0 virka '
        u'l\xedka, setja \xedslensku inn.')
    self.assertEqual(event.message, expected_message)

    event = events[1]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-12-26 19:11:57')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.sender, u'secd')
    self.assertEqual(event.sender_pid, 11)
    self.assertEqual(event.facility, u'serverxpc')
    self.assertEqual(event.security_api, u'SOSCCThisDeviceIsInCircle')
    self.assertEqual(event.caller, u'unknown')
    self.assertEqual(event.level, u'Notice')

    event = events[2]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-12-26 19:11:58')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.sender, u'secd')
    self.assertEqual(event.sender_pid, 111)
    self.assertEqual(event.facility, u'user')
    self.assertEqual(event.security_api, u'unknown')
    self.assertEqual(event.caller, u'unknown')
    self.assertEqual(event.level, u'Debug')

    event = events[3]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-12-26 19:11:59')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.sender, u'secd')
    self.assertEqual(event.sender_pid, 1111)
    self.assertEqual(event.facility, u'user')
    self.assertEqual(event.security_api, u'SOSCCThisDeviceIsInCircle')
    self.assertEqual(event.caller, u'C0x7fff872fa482')
    self.assertEqual(event.level, u'Error')

    event = events[4]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-12-06 19:11:01')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.sender, u'secd')
    self.assertEqual(event.sender_pid, 1)
    self.assertEqual(event.facility, u'user')
    self.assertEqual(event.security_api, u'unknown')
    self.assertEqual(event.caller, u'unknown')
    self.assertEqual(event.level, u'Error')
    self.assertEqual(event.message, u'')

    event = events[5]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-12-06 19:11:02')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.sender, u'secd')
    self.assertEqual(event.sender_pid, 11111)
    self.assertEqual(event.facility, u'user')
    self.assertEqual(event.security_api, u'SOSCCThisDeviceIsInCircle')
    self.assertEqual(event.caller, u'C0x7fff872fa482 F0x106080db0')
    self.assertEqual(event.level, u'Error')
    self.assertEqual(event.message, u'')

    event = events[6]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-12-31 23:59:59')
    self.assertEqual(event.timestamp, expected_timestamp)

    event = events[7]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-03-01 00:00:01')
    self.assertEqual(event.timestamp, expected_timestamp)

    # Repeated line.
    event = events[8]
    expected_message = u'Repeated 3 times: Happy new year!'
    self.assertEqual(event.message, expected_message)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for Basic Security Module (BSM) file parser."""

import unittest

from plaso.formatters import bsm  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import bsm

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class MacOSXBSMParserTest(test_lib.ParserTestCase):
  """Tests for Basic Security Module (BSM) file parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'apple.bsm'])
  def testParse(self):
    """Tests the Parse function on a Mac OS X BSM file."""
    parser = bsm.BSMParser()
    knowledge_base_values = {u'guessed_os': u'MacOSX'}
    storage_writer = self._ParseFile(
        [u'apple.bsm'], parser,
        knowledge_base_values=knowledge_base_values)

    self.assertEqual(storage_writer.number_of_events, 54)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.assertEqual(event.data_type, u'mac:bsm:event')

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-04 18:36:20.000381')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.event_type, u'audit crash recovery (45029)')

    expected_extra_tokens = {
        u'BSM_TOKEN_PATH': u'/var/audit/20131104171720.crash_recovery',
        u'BSM_TOKEN_RETURN32': {
            u'call_status': 0,
            u'error': u'Success',
            u'token_status': 0},
        u'BSM_TOKEN_TEXT': u'launchctl::Audit recovery',
        u'BSM_TOKEN_TRAILER': 104
    }

    self.assertEqual(event.extra_tokens, expected_extra_tokens)

    expected_return_value = {
        u'call_status': 0,
        u'error': u'Success',
        u'token_status': 0
    }
    self.assertEqual(event.return_value, expected_return_value)

    event = events[15]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-04 18:36:26.000171')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.event_type, u'user authentication (45023)')

    expected_extra_tokens = {
        u'BSM_TOKEN_RETURN32': {
            u'call_status': 5000,
            u'error': u'Unknown',
            u'token_status': 255},

        u'BSM_TOKEN_SUBJECT32': {
            u'aid': 4294967295,
            u'egid': 92,
            u'euid': 92,
            u'gid': 92,
            u'pid': 143,
            u'session_id': 100004,
            u'terminal_ip': '0.0.0.0',
            u'terminal_port': 143,
            u'uid': 92},

        u'BSM_TOKEN_TEXT': (u'Verify password for record type Users '
                            u'\'moxilo\' node \'/Local/Default\''),

        u'BSM_TOKEN_TRAILER': 140
    }

    self.assertEqual(event.extra_tokens, expected_extra_tokens)

    expected_return_value = {
        u'call_status': 5000,
        u'error': u'Unknown',
        u'token_status': 255
    }
    self.assertEqual(event.return_value, expected_return_value)

    event = events[31]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-04 18:36:26.000530')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.event_type, u'SecSrvr AuthEngine (45025)')
    expected_extra_tokens = {
        u'BSM_TOKEN_RETURN32': {
            u'call_status': 0,
            u'error': u'Success',
            u'token_status': 0},
        u'BSM_TOKEN_SUBJECT32': {
            u'aid': 4294967295,
            u'egid': 0,
            u'euid': 0,
            u'gid': 0,
            u'pid': 67,
            u'session_id': 100004,
            u'terminal_ip': '0.0.0.0',
            u'terminal_port': 67,
            u'uid': 0},
        u'BSM_TOKEN_TEXT': u'system.login.done',
        u'BSM_TOKEN_TRAILER': 110
    }
    self.assertEqual(event.extra_tokens, expected_extra_tokens)

    expected_return_value = {
        u'call_status': 0,
        u'error': u'Success',
        u'token_status': 0
    }
    self.assertEqual(event.return_value, expected_return_value)

    event = events[50]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-04 18:37:36.000399')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.event_type, u'session end (44903)')

    expected_extra_tokens = {
        u'BSM_TOKEN_ARGUMENT32': {
            u'is': 12288,
            u'num_arg': 3,
            u'string': u'am_failure'},
        u'BSM_TOKEN_ARGUMENT64': {
            u'is': 0,
            u'num_arg': 1,
            u'string': u'sflags'},
        u'BSM_TOKEN_RETURN32': {
            u'call_status': 0,
            u'error': u'Success',
            u'token_status': 0},
        u'BSM_TOKEN_SUBJECT32': {
            u'aid': 4294967295,
            u'egid': 0,
            u'euid': 0,
            u'gid': 0,
            u'pid': 0,
            u'session_id': 100015,
            u'terminal_ip': '0.0.0.0',
            u'terminal_port': 0,
            u'uid': 0},
        u'BSM_TOKEN_TRAILER': 125
    }
    self.assertEqual(event.extra_tokens, expected_extra_tokens)

    expected_return_value = {
        u'call_status': 0,
        u'error': u'Success',
        u'token_status': 0
    }
    self.assertEqual(event.return_value, expected_return_value)


class OpenBSMParserTest(test_lib.ParserTestCase):
  """Tests for Basic Security Module (BSM) file parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'openbsm.bsm'])
  def testParse(self):
    """Tests the Parse function on a "generic" BSM file."""
    parser = bsm.BSMParser()
    knowledge_base_values = {u'guessed_os': u'openbsm'}
    storage_writer = self._ParseFile(
        [u'openbsm.bsm'],
        parser,
        knowledge_base_values=knowledge_base_values)

    self.assertEqual(storage_writer.number_of_events, 50)

    events = list(storage_writer.GetEvents())

    expected_extra_tokens = [
        {u'BSM_TOKEN_ARGUMENT32': {
            u'is': 2882400000,
            u'num_arg': 3,
            u'string': u'test_arg32_token'},
         u'BSM_TOKEN_TRAILER': 50},
        {u'BSM_TOKEN_DATA':{
            u'data': u'SomeData',
            u'format': u'String'},
         u'BSM_TOKEN_TRAILER': 39},
        {u'BSM_TOKEN_FILE': {
            u'string': u'test',
            u'timestamp': '1970-01-01 20:42:45'},
         u'BSM_TOKEN_TRAILER': 41},
        {u'BSM_TOKEN_ADDR': '192.168.100.15',
         u'BSM_TOKEN_TRAILER': 30},
        {u'BSM_TOKEN_TRAILER': 46,
         u'IPv4_Header': u'0x400000145478000040010000c0a8649bc0a86e30]'},
        {u'BSM_TOKEN_IPC': {
            u'object_id': 305419896,
            u'object_type': 1},
         u'BSM_TOKEN_TRAILER': 31},
        {u'BSM_TOKEN_PORT': 20480,
         u'BSM_TOKEN_TRAILER': 28},
        {u'BSM_TOKEN_OPAQUE': u'aabbccdd',
         u'BSM_TOKEN_TRAILER': 32},
        {u'BSM_TOKEN_PATH': u'/test/this/is/a/test',
         u'BSM_TOKEN_TRAILER': 49},
        {u'BSM_TOKEN_PROCESS32': {
            u'aid': 305419896,
            u'egid': 591751049,
            u'euid': 19088743,
            u'gid': 159868227,
            u'pid': 321140038,
            u'session_id': 2542171492,
            u'terminal_ip': '127.0.0.1',
            u'terminal_port': 374945606,
            u'uid': 2557891634},
         u'BSM_TOKEN_TRAILER': 62},
        {u'BSM_TOKEN_PROCESS64': {
            u'aid': 305419896,
            u'egid': 591751049,
            u'euid': 19088743,
            u'gid': 159868227,
            u'pid': 321140038,
            u'session_id': 2542171492,
            u'terminal_ip': '127.0.0.1',
            u'terminal_port': 374945606,
            u'uid': 2557891634},
         u'BSM_TOKEN_TRAILER': 66},
        {u'BSM_TOKEN_RETURN32': {
            u'call_status': 305419896,
            u'error': u'Invalid argument',
            u'token_status': 22},
         u'BSM_TOKEN_TRAILER': 31},
        {u'BSM_TOKEN_SEQUENCE': 305419896,
         u'BSM_TOKEN_TRAILER': 30},
        {u'BSM_TOKEN_AUT_SOCKINET32_EX':{
            u'from': '127.0.0.1',
            u'from_port': 0,
            u'to': '127.0.0.1',
            u'to_port': 0},
         u'BSM_TOKEN_TRAILER': 44},
        {u'BSM_TOKEN_SUBJECT32': {
            u'aid': 305419896,
            u'egid': 591751049,
            u'euid': 19088743,
            u'gid': 159868227,
            u'pid': 321140038,
            u'session_id': 2542171492,
            u'terminal_ip': '127.0.0.1',
            u'terminal_port': 374945606,
            u'uid': 2557891634},
         u'BSM_TOKEN_TRAILER': 62},
        {u'BSM_TOKEN_SUBJECT32_EX': {
            u'aid': 305419896,
            u'egid': 591751049,
            u'euid': 19088743,
            u'gid': 159868227,
            u'pid': 321140038,
            u'session_id': 2542171492,
            u'terminal_ip': 'fe80::1',
            u'terminal_port': 374945606,
            u'uid': 2557891634},
         u'BSM_TOKEN_TRAILER': 78},
        {u'BSM_TOKEN_TEXT': u'This is a test.',
         u'BSM_TOKEN_TRAILER': 44},
        {u'BSM_TOKEN_TRAILER': 37,
         u'BSM_TOKEN_ZONENAME': u'testzone'},
        {u'BSM_TOKEN_RETURN32': {
            u'call_status': 4294967295,
            u'error':
            u'Argument list too long',
            u'token_status': 7},
         u'BSM_TOKEN_TRAILER': 31}
    ]

    for event_index in range(0, 19):
      event = events[event_index]
      expected_extra_tokens_dict = expected_extra_tokens[event_index]
      extra_tokens_dict = getattr(event, u'extra_tokens', {})
      self.assertDictContains(extra_tokens_dict, expected_extra_tokens_dict)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""SQLite database plugin related functions and classes for testing."""

from dfvfs.lib import definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.containers import sessions
from plaso.parsers import sqlite
from plaso.storage import fake_storage

from tests.parsers import test_lib


class SQLitePluginTestCase(test_lib.ParserTestCase):
  """The unit test case for SQLite database plugins."""

  def _ParseDatabaseFileWithPlugin(
      self, path_segments, plugin, cache=None,
      knowledge_base_values=None, wal_path=None):
    """Parses a file as a SQLite database with a specific plugin.

    Args:
      path_segments: a list of strings containinge the path segments inside
                     the test data directory.
      plugin: The plugin object that is used to extract an event
                     generator.
      cache: optional cache object (instance of SQLiteCache).
      knowledge_base_values: optional dict containing the knowledge base
                             values.
      wal_path: optional string containing the path to the SQLite WAL file.

    Returns:
      A storage writer object (instance of FakeStorageWriter).
    """
    session = sessions.Session()
    storage_writer = fake_storage.FakeStorageWriter(session)
    storage_writer.Open()

    file_entry = self._GetTestFileEntry(path_segments)
    parser_mediator = self._CreateParserMediator(
        storage_writer, file_entry=file_entry,
        knowledge_base_values=knowledge_base_values)

    parser_mediator.SetFileEntry(file_entry)

    wal_file_entry = None
    if wal_path:
      wal_path_spec = path_spec_factory.Factory.NewPathSpec(
          definitions.TYPE_INDICATOR_OS, location=wal_path)
      wal_file_entry = path_spec_resolver.Resolver.OpenFileEntry(wal_path_spec)

    # AppendToParserChain needs to be run after SetFileEntry.
    parser_mediator.AppendToParserChain(plugin)

    database = sqlite.SQLiteDatabase(file_entry.name)
    database_wal = None

    file_object = file_entry.GetFileObject()
    try:
      database.Open(file_object)
      if wal_file_entry:
        database_wal = sqlite.SQLiteDatabase(file_entry.name)
        wal_file_object = wal_file_entry.GetFileObject()
        # Seek file_object to 0 so we can re-open the database with WAL file.
        file_object.seek(0)
        try:
          database_wal.Open(file_object, wal_file_object=wal_file_object)
        finally:
          wal_file_object.close()
    finally:
      file_object.close()

    try:
      plugin.Process(
          parser_mediator, cache=cache, database=database,
          database_wal=database_wal, wal_file_entry=wal_file_entry)
    finally:
      database.Close()
      if database_wal:
        database_wal.Close()

    return storage_writer



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the OXML parser."""

import unittest

from plaso.formatters import oxml  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import oxml

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class OXMLTest(test_lib.ParserTestCase):
  """Tests for the OXML parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'Document.docx'])
  def testParse(self):
    """Tests the Parse function."""
    parser = oxml.OpenXMLParser()
    storage_writer = self._ParseFile([u'Document.docx'], parser)

    self.assertEqual(storage_writer.number_of_events, 2)

    events = list(storage_writer.GetEvents())

    event = events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-11-07 23:29:00')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    event = events[1]

    self.assertEqual(event.number_of_characters, u'13')
    self.assertEqual(event.total_time, u'1385')
    self.assertEqual(event.number_of_characters_with_spaces, u'14')
    self.assertEqual(event.i4, u'1')
    self.assertEqual(event.app_version, u'14.0000')
    self.assertEqual(event.number_of_lines, u'1')
    self.assertEqual(event.scale_crop, u'false')
    self.assertEqual(event.number_of_pages, u'1')
    self.assertEqual(event.number_of_words, u'2')
    self.assertEqual(event.links_up_to_date, u'false')
    self.assertEqual(event.number_of_paragraphs, u'1')
    self.assertEqual(event.doc_security, u'0')
    self.assertEqual(event.hyperlinks_changed, u'false')
    self.assertEqual(event.revision_number, u'3')
    self.assertEqual(event.last_saved_by, u'Nides')
    self.assertEqual(event.author, u'Nides')
    self.assertEqual(
        event.creating_app, u'Microsoft Office Word')
    self.assertEqual(event.template, u'Normal.dotm')

    expected_message = (
        u'Creating App: Microsoft Office Word '
        u'App version: 14.0000 '
        u'Last saved by: Nides '
        u'Author: Nides '
        u'Revision number: 3 '
        u'Template: Normal.dotm '
        u'Number of pages: 1 '
        u'Number of words: 2 '
        u'Number of characters: 13 '
        u'Number of characters with spaces: 14 '
        u'Number of lines: 1 '
        u'Hyperlinks changed: false '
        u'Links up to date: false '
        u'Scale crop: false')
    expected_short_message = (
        u'Author: Nides')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Front-end related functions and classes for testing."""

import io

from plaso.analysis import interface as analysis_interface


# TODO: replace by cli_test_lib.TestOutputWriter
class StringIOOutputWriter(object):
  """Class that implements a StringIO output writer."""

  def __init__(self):
    """Initialize the string output writer."""
    super(StringIOOutputWriter, self).__init__()
    self._string_io = io.StringIO()

    # Make the output writer compatible with a filehandle interface.
    self.write = self.Write

  def flush(self):
    """Flush the internal buffer."""
    self._string_io.flush()

  def GetValue(self):
    """Returns the write buffer from the output writer."""
    return self._string_io.getvalue()

  def GetLine(self):
    """Returns a single line read from the output buffer."""
    return self._string_io.readline()

  def SeekToBeginning(self):
    """Seeks the output buffer to the beginning of the buffer."""
    self._string_io.seek(0)

  def Write(self, string):
    """Writes a string to the StringIO object."""
    self._string_io.write(string)


class TestAnalysisPlugin(analysis_interface.AnalysisPlugin):
  """Test analysis plugin."""

  NAME = u'test_analysis_plugin'

  def CompileReport(self, unused_analysis_mediator):
    """Compiles a report of the analysis.

    After the plugin has received every copy of an event to
    analyze this function will be called so that the report
    can be assembled.

    Args:
      analysis_mediator: The analysis mediator object (instance of
                         AnalysisMediator).

    Returns:
      The analysis report (instance of AnalysisReport).
    """
    return

  def ExamineEvent(
      self, unused_analysis_mediator, unused_event_object, **unused_kwargs):
    """Analyzes an event object.

    Args:
      analysis_mediator: The analysis mediator object (instance of
                         AnalysisMediator).
      event_object: An event object (instance of EventObject).
    """
    return



# -*- coding: utf-8 -*-
"""The shell item event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class ShellItemFileEntryEventFormatter(interface.ConditionalEventFormatter):
  """Formatter for a shell item file entry event."""

  DATA_TYPE = u'windows:shell_item:file_entry'

  FORMAT_STRING_PIECES = [
      u'Name: {name}',
      u'Long name: {long_name}',
      u'Localized name: {localized_name}',
      u'NTFS file reference: {file_reference}',
      u'Shell item path: {shell_item_path}',
      u'Origin: {origin}']

  FORMAT_STRING_SHORT_PIECES = [
      u'Name: {file_entry_name}',
      u'NTFS file reference: {file_reference}',
      u'Origin: {origin}']

  SOURCE_LONG = u'File entry shell item'
  SOURCE_SHORT = u'FILE'

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    event_values[u'file_entry_name'] = event_values.get(u'long_name', None)
    if not event_values[u'file_entry_name']:
      event_values[u'file_entry_name'] = event_values.get(u'name', None)

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(ShellItemFileEntryEventFormatter)



# -*- coding: utf-8 -*-
"""Plugin to parse .automaticDestinations-ms OLECF files."""

import re
import uuid

import construct

from dfdatetime import filetime as dfdatetime_filetime
from dfdatetime import semantic_time as dfdatetime_semantic_time
from dfdatetime import uuid_time as dfdatetime_uuid_time

from plaso.containers import time_events
from plaso.containers import windows_events
from plaso.lib import binary
from plaso.lib import errors
from plaso.lib import definitions
from plaso.parsers import olecf
from plaso.parsers import winlnk
from plaso.parsers.olecf_plugins import interface


class AutomaticDestinationsDestListEntryEvent(time_events.DateTimeValuesEvent):
  """Convenience class for an .automaticDestinations-ms DestList entry event.

  Attributes:
    birth_droid_file_identifier (str): birth droid file identifier.
    birth_droid_volume_identifier (str): birth droid volume identifier.
    droid_file_identifier (str): droid file identifier.
    droid_volume_identifier (str): droid volume identifier.
    entry_number (int): DestList entry number.
    path (str): path.
    pin_status (int): pin status.
    offset (int): offset of the DestList entry relative to the start of
        the DestList stream.
  """

  DATA_TYPE = u'olecf:dest_list:entry'

  def __init__(
      self, date_time, date_time_description, entry_offset, dest_list_entry,
      droid_volume_identifier, droid_file_identifier,
      birth_droid_volume_identifier, birth_droid_file_identifier):
    """Initializes an event.

    Args:
      date_time (dfdatetime.DateTimeValues): date and time values.
      date_time_description (str): description of the meaning of the date
          and time values.
      entry_offset (int): offset of the DestList entry relative to the start of
          the DestList stream.
      droid_volume_identifier (str): droid volume identifier.
      droid_file_identifier (str): droid file identifier.
      birth_droid_volume_identifier (str): birth droid volume identifier.
      birth_droid_file_identifier (str): birth droid file identifier.
      dest_list_entry (construct.Struct): DestList entry.
    """
    # TODO: move to parser plugin.
    hostname = binary.ByteStreamCopyToString(
        dest_list_entry.hostname, codepage=u'ascii')
    path = binary.UTF16StreamCopyToString(dest_list_entry.path)

    super(AutomaticDestinationsDestListEntryEvent, self).__init__(
        date_time, date_time_description)
    self.birth_droid_file_identifier = birth_droid_file_identifier
    self.birth_droid_volume_identifier = birth_droid_volume_identifier
    self.droid_file_identifier = droid_file_identifier
    self.droid_volume_identifier = droid_volume_identifier
    self.entry_number = dest_list_entry.entry_number
    self.hostname = hostname
    self.offset = entry_offset
    self.path = path
    self.pin_status = dest_list_entry.pin_status


class AutomaticDestinationsOLECFPlugin(interface.OLECFPlugin):
  """Plugin that parses an .automaticDestinations-ms OLECF file."""

  NAME = u'olecf_automatic_destinations'
  DESCRIPTION = u'Parser for *.automaticDestinations-ms OLECF files.'

  REQUIRED_ITEMS = frozenset([u'DestList'])

  _RE_LNK_ITEM_NAME = re.compile(r'^[1-9a-f][0-9a-f]*$')

  # We cannot use the parser registry here since winlnk could be disabled.
  # TODO: see if there is a more elegant solution for this.
  _WINLNK_PARSER = winlnk.WinLnkParser()

  _DEST_LIST_STREAM_HEADER = construct.Struct(
      u'dest_list_stream_header',
      construct.ULInt32(u'format_version'),
      construct.ULInt32(u'number_of_entries'),
      construct.ULInt32(u'number_of_pinned_entries'),
      construct.Padding(4),
      construct.ULInt32(u'last_entry_number'),
      construct.Padding(4),
      construct.ULInt32(u'last_revision_number'),
      construct.Padding(4))

  _DEST_LIST_STREAM_HEADER_SIZE = _DEST_LIST_STREAM_HEADER.sizeof()

  # Using Construct's utf-16 encoding here will create strings with their
  # end-of-string characters exposed. Instead the strings are read as
  # binary strings and converted using ReadUTF16().
  _DEST_LIST_STREAM_ENTRY_V1 = construct.Struct(
      u'dest_list_stream_entry_v1',
      construct.Padding(8),
      construct.Bytes(u'droid_volume_identifier', 16),
      construct.Bytes(u'droid_file_identifier', 16),
      construct.Bytes(u'birth_droid_volume_identifier', 16),
      construct.Bytes(u'birth_droid_file_identifier', 16),
      construct.String(u'hostname', 16),
      construct.ULInt32(u'entry_number'),
      construct.Padding(8),
      construct.ULInt64(u'last_modification_time'),
      construct.ULInt32(u'pin_status'),
      construct.ULInt16(u'path_size'),
      construct.String(u'path', lambda ctx: ctx.path_size * 2))

  _DEST_LIST_STREAM_ENTRY_V3 = construct.Struct(
      u'dest_list_stream_entry_v3',
      construct.Padding(8),
      construct.Bytes(u'droid_volume_identifier', 16),
      construct.Bytes(u'droid_file_identifier', 16),
      construct.Bytes(u'birth_droid_volume_identifier', 16),
      construct.Bytes(u'birth_droid_file_identifier', 16),
      construct.String(u'hostname', 16),
      construct.ULInt32(u'entry_number'),
      construct.Padding(8),
      construct.ULInt64(u'last_modification_time'),
      construct.ULInt32(u'pin_status'),
      construct.Padding(16),
      construct.ULInt16(u'path_size'),
      construct.String(u'path', lambda ctx: ctx.path_size * 2),
      construct.Padding(4))

  def _ParseDistributedTrackingIdentifier(
      self, parser_mediator, uuid_data, origin):
    """Extracts data from a Distributed Tracking identifier.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      uuid_data (bytes): UUID data of the Distributed Tracking identifier.
      origin (str): origin of the event (event source).

    Returns:
      str: UUID string of the Distributed Tracking identifier.
    """
    uuid_object = uuid.UUID(bytes_le=uuid_data)

    if uuid_object.version == 1:
      event_data = windows_events.WindowsDistributedLinkTrackingEventData(
          uuid_object, origin)
      date_time = dfdatetime_uuid_time.UUIDTime(timestamp=uuid_object.time)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_CREATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

    return u'{{{0!s}}}'.format(uuid_object)

  def ParseDestList(self, parser_mediator, olecf_item):
    """Parses the DestList OLECF item.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      olecf_item (pyolecf.item): OLECF item.
    """
    try:
      header = self._DEST_LIST_STREAM_HEADER.parse_stream(olecf_item)
    except (IOError, construct.FieldError) as exception:
      raise errors.UnableToParseFile(
          u'Unable to parse DestList header with error: {0:s}'.format(
              exception))

    if header.format_version not in (1, 3, 4):
      parser_mediator.ProduceExtractionError(
          u'unsupported format version: {0:d}.'.format(header.format_version))

    if header.format_version == 1:
      dest_list_stream_entry = self._DEST_LIST_STREAM_ENTRY_V1
    elif header.format_version in (3, 4):
      dest_list_stream_entry = self._DEST_LIST_STREAM_ENTRY_V3

    entry_offset = olecf_item.get_offset()
    while entry_offset < olecf_item.size:
      try:
        entry = dest_list_stream_entry.parse_stream(olecf_item)
      except (IOError, construct.FieldError) as exception:
        raise errors.UnableToParseFile(
            u'Unable to parse DestList entry with error: {0:s}'.format(
                exception))

      if not entry:
        break

      display_name = u'DestList entry at offset: 0x{0:08x}'.format(entry_offset)

      try:
        droid_volume_identifier = self._ParseDistributedTrackingIdentifier(
            parser_mediator, entry.droid_volume_identifier, display_name)

      except (TypeError, ValueError) as exception:
        droid_volume_identifier = u''
        parser_mediator.ProduceExtractionError(
            u'unable to read droid volume identifier with error: {0:s}'.format(
                exception))

      try:
        droid_file_identifier = self._ParseDistributedTrackingIdentifier(
            parser_mediator, entry.droid_file_identifier, display_name)

      except (TypeError, ValueError) as exception:
        droid_file_identifier = u''
        parser_mediator.ProduceExtractionError(
            u'unable to read droid file identifier with error: {0:s}'.format(
                exception))

      try:
        birth_droid_volume_identifier = (
            self._ParseDistributedTrackingIdentifier(
                parser_mediator, entry.birth_droid_volume_identifier,
                display_name))

      except (TypeError, ValueError) as exception:
        birth_droid_volume_identifier = u''
        parser_mediator.ProduceExtractionError((
            u'unable to read birth droid volume identifier with error: '
            u'{0:s}').format(
                exception))

      try:
        birth_droid_file_identifier = self._ParseDistributedTrackingIdentifier(
            parser_mediator, entry.birth_droid_file_identifier, display_name)

      except (TypeError, ValueError) as exception:
        birth_droid_file_identifier = u''
        parser_mediator.ProduceExtractionError((
            u'unable to read birth droid file identifier with error: '
            u'{0:s}').format(
                exception))

      if entry.last_modification_time == 0:
        date_time = dfdatetime_semantic_time.SemanticTime(u'Not set')
      else:
        date_time = dfdatetime_filetime.Filetime(
            timestamp=entry.last_modification_time)

      event = AutomaticDestinationsDestListEntryEvent(
          date_time, definitions.TIME_DESCRIPTION_MODIFICATION, entry_offset,
          entry, droid_volume_identifier, droid_file_identifier,
          birth_droid_volume_identifier, birth_droid_file_identifier)
      parser_mediator.ProduceEvent(event)

      entry_offset = olecf_item.get_offset()

  def Process(self, parser_mediator, root_item=None, **kwargs):
    """Parses an OLECF file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      root_item (Optional[pyolecf.item]): root item of the OLECF file.

    Raises:
      ValueError: If the root_item is not set.
    """
    # This will raise if unhandled keyword arguments are passed.
    super(AutomaticDestinationsOLECFPlugin, self).Process(
        parser_mediator, **kwargs)

    if not root_item:
      raise ValueError(u'Root item not set.')

    for item in root_item.sub_items:
      if item.name == u'DestList':
        self.ParseDestList(parser_mediator, item)

      elif self._RE_LNK_ITEM_NAME.match(item.name):
        display_name = parser_mediator.GetDisplayName()
        if display_name:
          display_name = u'{0:s} # {1:s}'.format(display_name, item.name)
        else:
          display_name = u'# {0:s}'.format(item.name)

        self._WINLNK_PARSER.Parse(
            parser_mediator, item, display_name=display_name)

        # TODO: check for trailing data?


olecf.OLECFParser.RegisterPlugin(AutomaticDestinationsOLECFPlugin)



# -*- coding: utf-8 -*-
"""The Viper analysis plugin CLI arguments helper."""

from plaso.analysis import viper
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.lib import errors


class ViperAnalysisArgumentsHelper(interface.ArgumentsHelper):
  """Viper analysis plugin CLI arguments helper."""

  NAME = u'viper_analysis'
  CATEGORY = u'analysis'
  DESCRIPTION = u'Argument helper for the Viper analysis plugin.'

  _DEFAULT_HASH = u'sha256'
  _DEFAULT_HOST = u'localhost'
  _DEFAULT_PORT = 8080
  _DEFAULT_PROTOCOL = u'http'

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments the helper supports to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        u'--viper-hash', u'--viper_hash', dest=u'viper_hash', type=str,
        action='store', choices=viper.ViperAnalyzer.SUPPORTED_HASHES,
        default=cls._DEFAULT_HASH, metavar=u'HASH', help=(
            u'Type of hash to use to query the Viper server, the default is: '
            u'{0:s}. Supported options: {1:s}').format(
                cls._DEFAULT_HASH, u', '.join(
                    viper.ViperAnalyzer.SUPPORTED_HASHES)))

    argument_group.add_argument(
        u'--viper-host', u'--viper_host', dest=u'viper_host', type=str,
        action='store', default=cls._DEFAULT_HOST, metavar=u'HOST',
        help=(
            u'Hostname of the Viper server to query, the default is: '
            u'{0:s}'.format(cls._DEFAULT_HOST)))

    argument_group.add_argument(
        u'--viper-port', u'--viper_port', dest=u'viper_port', type=int,
        action='store', default=cls._DEFAULT_PORT, metavar=u'PORT', help=(
            u'Port of the Viper server to query, the default is: {0:d}.'.format(
                cls._DEFAULT_PORT)))

    argument_group.add_argument(
        u'--viper-protocol', u'--viper_protocol', dest=u'viper_protocol',
        type=str, choices=viper.ViperAnalyzer.SUPPORTED_PROTOCOLS,
        action='store', default=cls._DEFAULT_PROTOCOL, metavar=u'PROTOCOL',
        help=(
            u'Protocol to use to query Viper, the default is: {0:s}. '
            u'Supported options: {1:s}').format(
                cls._DEFAULT_PROTOCOL, u', '.join(
                    viper.ViperAnalyzer.SUPPORTED_PROTOCOLS)))

  @classmethod
  def ParseOptions(cls, options, analysis_plugin):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      analysis_plugin (ViperAnalysisPlugin): analysis plugin to configure.

    Raises:
      BadConfigObject: when the output module object is of the wrong type.
      BadConfigOption: when unable to connect to Viper instance.
    """
    if not isinstance(analysis_plugin, viper.ViperAnalysisPlugin):
      raise errors.BadConfigObject(
          u'Analysis plugin is not an instance of ViperAnalysisPlugin')

    lookup_hash = cls._ParseStringOption(
        options, u'viper_hash', default_value=cls._DEFAULT_HASH)
    analysis_plugin.SetLookupHash(lookup_hash)

    host = cls._ParseStringOption(
        options, u'viper_host', default_value=cls._DEFAULT_HOST)
    analysis_plugin.SetHost(host)

    port = cls._ParseNumericOption(
        options, u'viper_port', default_value=cls._DEFAULT_PORT)
    analysis_plugin.SetPort(port)

    protocol = cls._ParseStringOption(
        options, u'viper_protocol', default_value=cls._DEFAULT_PROTOCOL)
    protocol = protocol.lower().strip()
    analysis_plugin.SetProtocol(protocol)

    if not analysis_plugin.TestConnection():
      raise errors.BadConfigOption(
          u'Unable to connect to Viper {0:s}:{1:d}'.format(host, port))


manager.ArgumentHelperManager.RegisterHelper(ViperAnalysisArgumentsHelper)



# -*- coding: utf-8 -*-
"""Spotlight Volume Configuration plist plugin."""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import plist_event
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


__author__ = 'Joaquin Moreno Garijo (Joaquin.MorenoGarijo.2013@live.rhul.ac.uk)'


class SpotlightVolumePlugin(interface.PlistPlugin):
  """Basic plugin to extract the Spotlight Volume Configuration."""

  NAME = u'spotlight_volume'
  DESCRIPTION = u'Parser for Spotlight volume configuration plist files.'

  PLIST_PATH = u'VolumeConfiguration.plist'
  PLIST_KEYS = frozenset([u'Stores'])

  def GetEntries(self, parser_mediator, match=None, **unused_kwargs):
    """Extracts relevant Volume Configuration Spotlight entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    stores = match.get(u'Stores', {})
    for volume_name, volume in iter(stores.items()):
      datetime_value = volume.get(u'CreationDate', None)
      if not datetime_value:
        continue

      partial_path = volume[u'PartialPath']

      event_data = plist_event.PlistTimeEventData()
      event_data.desc = u'Spotlight Volume {0:s} ({1:s}) activated.'.format(
          volume_name, partial_path)
      event_data.key = u''
      event_data.root = u'/Stores'

      timestamp = timelib.Timestamp.FromPythonDatetime(datetime_value)
      date_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
          timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_WRITTEN)
      parser_mediator.ProduceEventWithEventData(event, event_data)


plist.PlistParser.RegisterPlugin(SpotlightVolumePlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Sleuthkit (TSK) bodyfile (or mactime) event formatter."""

import unittest

from plaso.formatters import mactime

from tests.formatters import test_lib


class MactimeFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the mactime event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = mactime.MactimeFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = mactime.MactimeFormatter()

    expected_attribute_names = [u'filename']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Google Chrome extension activity database event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class ChromeExtensionActivityEventFormatter(
    interface.ConditionalEventFormatter):
  """Formatter for a Chrome extension activity event."""

  DATA_TYPE = u'chrome:extension_activity:activity_log'

  FORMAT_STRING_PIECES = [
      u'Chrome extension: {extension_id}',
      u'Action type: {action_type}',
      u'Activity identifier: {activity_id}',
      u'Page URL: {page_url}',
      u'Page title: {page_title}',
      u'API name: {api_name}',
      u'Args: {args}',
      u'Other: {other}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{extension_id}',
      u'{api_name}',
      u'{args}']

  SOURCE_LONG = u'Chrome Extension Activity'
  SOURCE_SHORT = u'WEBHIST'

  # From:
  # https://chromium.googlesource.com/chromium/src.git/+/master/chrome/browser/extensions/activity_log/activity_actions.h
  _CHROME_ACTION_TYPES = {
      0 : "API call",
      1 : "API event callback",
      2 : "API action blocked",
      3 : "Content Script inserted",
      4 : "DOM access",
      5 : "DOM event",
      6 : "WebRequest",
      1001 : "Unspecified"
  }

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    action_type = event_values.get(u'action_type')
    event_values[u'action_type'] = u'%s (type %d)'%(
        self._CHROME_ACTION_TYPES.get(action_type, "unknown action_type"),
        action_type
    )

    return self._ConditionalFormatMessages(event_values)



manager.FormattersManager.RegisterFormatter(
    ChromeExtensionActivityEventFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Script to check for the availability and version of dependencies."""

from __future__ import print_function
import sys

# Change PYTHONPATH to include dependencies.
sys.path.insert(0, u'.')

import utils.dependencies  # pylint: disable=wrong-import-position


if __name__ == u'__main__':
  dependency_helper = utils.dependencies.DependencyHelper()

  if not dependency_helper.CheckDependencies():
    build_instructions_url = (
        u'https://github.com/log2timeline/plaso/wiki/Users-Guide')

    print(u'See: {0:s} on how to set up plaso.'.format(build_instructions_url))
    print(u'')



# -*- coding: utf-8 -*-
"""Analysis plugin to look up files in VirusTotal and tag events."""

from __future__ import unicode_literals

import logging

from plaso.analysis import interface
from plaso.analysis import manager
from plaso.lib import errors


class VirusTotalAnalyzer(interface.HTTPHashAnalyzer):
  """Class that analyzes file hashes by consulting VirusTotal."""

  _VIRUSTOTAL_API_REPORT_URL = (
      'https://www.virustotal.com/vtapi/v2/file/report')

  _EICAR_SHA256 = (
      '275a021bbfb6489e54d471899f7db9d1663fc695ec2fe2a2c4538aabf651fd0f')

  SUPPORTED_HASHES = ['md5', 'sha1', 'sha256']

  def __init__(self, hash_queue, hash_analysis_queue, **kwargs):
    """Initializes a VirusTotal analyzer.

    Args:
      hash_queue (Queue.queue): queue that contains hashes to be analyzed.
      hash_analysis_queue (Queue.queue): queue the analyzer will append
          HashAnalysis objects to.
    """
    super(VirusTotalAnalyzer, self).__init__(
        hash_queue, hash_analysis_queue, **kwargs)
    self._api_key = None
    self._checked_for_old_python_version = False

  def _QueryHashes(self, digests):
    """Queries VirusTotal for a specfic hashes.

    Args:
      digests (list[str]): hashes to look up.

    Returns:
      dict[str, object]: JSON response or None on error.
    """
    url_parameters = {'apikey': self._api_key, 'resource': ', '.join(digests)}

    try:
      json_response = self.MakeRequestAndDecodeJSON(
          self._VIRUSTOTAL_API_REPORT_URL, 'GET', params=url_parameters)
    except errors.ConnectionError as exception:
      json_response = None
      logging.error(u'Unable to query VirusTotal with error: {0:s}.'.format(
          exception))

    return json_response

  def Analyze(self, hashes):
    """Looks up hashes in VirusTotal using the VirusTotal HTTP API.

    The API is documented here:
      https://www.virustotal.com/en/documentation/public-api/

    Args:
      hashes (list[str]): hashes to look up.

    Returns:
      list[HashAnalysis]: analysis results.

    Raises:
      RuntimeError: If the VirusTotal API key has not been set.
    """
    if not self._api_key:
      raise RuntimeError('No API key specified for VirusTotal lookup.')

    hash_analyses = []

    json_response = self._QueryHashes(hashes) or []

    # VirusTotal returns a dictionary when a single hash is queried
    # and a list when multiple hashes are queried.
    if isinstance(json_response, dict):
      json_response = [json_response]

    for result in json_response:
      resource = result['resource']
      hash_analysis = interface.HashAnalysis(resource, result)
      hash_analyses.append(hash_analysis)

    return hash_analyses

  def SetAPIKey(self, api_key):
    """Sets the VirusTotal API key to use in queries.

    Args:
      api_key (str): VirusTotal API key
    """
    self._api_key = api_key

  def TestConnection(self):
    """Tests the connection to VirusTotal

    Returns:
      bool: True if VirusTotal is reachable.
    """
    json_response = self._QueryHashes([self._EICAR_SHA256])
    return json_response is not None


class VirusTotalAnalysisPlugin(interface.HashTaggingAnalysisPlugin):
  """An analysis plugin for looking up hashes in VirusTotal."""

  # TODO: Check if there are other file types worth checking VirusTotal for.
  DATA_TYPES = ['pe:compilation:compilation_time']

  URLS = ['https://virustotal.com']

  NAME = 'virustotal'

  _VIRUSTOTAL_NOT_PRESENT_RESPONSE_CODE = 0
  _VIRUSTOTAL_PRESENT_RESPONSE_CODE = 1
  _VIRUSTOTAL_ANALYSIS_PENDING_RESPONSE_CODE = -2

  def __init__(self):
    """Initializes a VirusTotal analysis plugin."""
    super(VirusTotalAnalysisPlugin, self).__init__(VirusTotalAnalyzer)
    self._api_key = None

  def EnableFreeAPIKeyRateLimit(self):
    """Configures Rate limiting for queries to VirusTotal.

    The default rate limit for free VirusTotal API keys is 4 requests per
    minute.
    """
    self._analyzer.hashes_per_batch = 4
    self._analyzer.wait_after_analysis = 60
    self._analysis_queue_timeout = self._analyzer.wait_after_analysis + 1

  def GenerateLabels(self, hash_information):
    """Generates a list of strings that will be used in the event tag.

    Args:
      hash_information (dict[str, object]): the JSON decoded contents of the
          result of a VirusTotal lookup, as produced by the VirusTotalAnalyzer.

    Returns:
      list[str]: strings describing the results from VirusTotal.
    """
    response_code = hash_information['response_code']
    if response_code == self._VIRUSTOTAL_NOT_PRESENT_RESPONSE_CODE:
      return ['virustotal_not_present']
    elif response_code == self._VIRUSTOTAL_PRESENT_RESPONSE_CODE:
      positives = hash_information['positives']
      if positives > 0:
        return ['virustotal_detections_{0:d}'.format(positives)]
      return ['virsutotal_no_detections']
    elif response_code == self._VIRUSTOTAL_ANALYSIS_PENDING_RESPONSE_CODE:
      return ['virustotal_analysis_pending']
    else:
      logging.error(
          'VirusTotal returned unknown response code {0!s}'.format(
              response_code))
      return ['virustotal_unknown_response_code_{0:d}'.format(response_code)]

  def SetAPIKey(self, api_key):
    """Sets the VirusTotal API key to use in queries.

    Args:
      api_key (str): VirusTotal API key
    """
    self._analyzer.SetAPIKey(api_key)

  def TestConnection(self):
    """Tests the connection to VirusTotal

    Returns:
      bool: True if VirusTotal is reachable.
    """
    return self._analyzer.TestConnection()


manager.AnalysisPluginManager.RegisterPlugin(VirusTotalAnalysisPlugin)
